@@ -12,24 +12,27 @@
 
 # This file handles all flask-restful resources for /v3/OS-FEDERATION
 
+import http.client
+
 import flask
 import flask_restful
-import http.client
+from oslo_log import log
 from oslo_serialization import jsonutils
 
 from keystone.api._shared import authentication
 from keystone.api._shared import json_home_relations
+from keystone.api import validation
 from keystone.common import provider_api
 from keystone.common import rbac_enforcer
 from keystone.common import render_token
-from keystone.common import validation
+from keystone.common import validation as ks_validation
 import keystone.conf
 from keystone import exception
 from keystone.federation import schema
 from keystone.federation import utils
 from keystone.server import flask as ks_flask
 
-
+LOG = log.getLogger(__name__)
 CONF = keystone.conf.CONF
 ENFORCER = rbac_enforcer.RBACEnforcer
 PROVIDERS = provider_api.ProviderAPIs
@@ -40,8 +43,11 @@ _build_resource_relation = json_home_relations.os_federation_resource_rel_func
 
 IDP_ID_PARAMETER_RELATION = _build_param_relation(parameter_name='idp_id')
 PROTOCOL_ID_PARAMETER_RELATION = _build_param_relation(
-    parameter_name='protocol_id')
-SP_ID_PARAMETER_RELATION = _build_param_relation(parameter_name='sp_id')
+    parameter_name='protocol_id'
+)
+SERVICE_PROVIDER_ID_PARAMETER_RELATION = _build_param_relation(
+    parameter_name='service_provider_id'
+)
 
 
 def _combine_lists_uniquely(a, b):
@@ -73,10 +79,17 @@ class IdentityProvidersResource(_ResourceBase):
     collection_key = 'identity_providers'
     member_key = 'identity_provider'
     api_prefix = '/OS-FEDERATION'
-    _public_parameters = frozenset(['id', 'enabled', 'description',
-                                    'remote_ids', 'links', 'domain_id',
-                                    'authorization_ttl'
-                                    ])
+    _public_parameters = frozenset(
+        [
+            'id',
+            'enabled',
+            'description',
+            'remote_ids',
+            'links',
+            'domain_id',
+            'authorization_ttl',
+        ]
+    )
     _id_path_param_name_override = 'idp_id'
 
     @staticmethod
@@ -89,33 +102,28 @@ class IdentityProvidersResource(_ResourceBase):
         """
         base_path = ref['links'].get('self')
         if base_path is None:
-            base_path = '/'.join(ks_flask.base_url(path='/%s' % ref['id']))
+            base_path = '/'.join(
+                ks_flask.base_url(path='/{}'.format(ref['id']))
+            )
 
         for name in ['protocols']:
             ref['links'][name] = '/'.join([base_path, name])
 
-    def get(self, idp_id=None):
-        if idp_id is not None:
-            return self._get_idp(idp_id)
-        return self._list_idps()
-
-    def _get_idp(self, idp_id):
-        """Get an IDP resource.
-
-        GET/HEAD /OS-FEDERATION/identity_providers/{idp_id}
-        """
-        ENFORCER.enforce_call(action='identity:get_identity_provider')
-        ref = PROVIDERS.federation_api.get_idp(idp_id)
-        return self.wrap_member(ref)
-
-    def _list_idps(self):
+    @validation.request_query_schema(
+        schema.identity_provider_index_request_query
+    )
+    @validation.response_body_schema(
+        schema.identity_provider_index_response_body
+    )
+    def get(self):
         """List all identity providers.
 
         GET/HEAD /OS-FEDERATION/identity_providers
         """
         filters = ['id', 'enabled']
-        ENFORCER.enforce_call(action='identity:list_identity_providers',
-                              filters=filters)
+        ENFORCER.enforce_call(
+            action='identity:list_identity_providers', filters=filters
+        )
         hints = self.build_driver_hints(filters)
         refs = PROVIDERS.federation_api.list_idps(hints=hints)
         refs = [self.filter_params(r) for r in refs]
@@ -125,6 +133,41 @@ class IdentityProvidersResource(_ResourceBase):
             self._add_related_links(r)
         return collection
 
+
+class IdentityProviderResource(_ResourceBase):
+    collection_key = 'identity_providers'
+    member_key = 'identity_provider'
+    api_prefix = '/OS-FEDERATION'
+    _public_parameters = frozenset(
+        [
+            'id',
+            'enabled',
+            'description',
+            'remote_ids',
+            'links',
+            'domain_id',
+            'authorization_ttl',
+        ]
+    )
+    _id_path_param_name_override = 'idp_id'
+
+    @validation.request_query_schema(schema.identity_provider_request_query)
+    @validation.response_body_schema(
+        schema.identity_provider_index_response_body
+    )
+    def get(self, idp_id=None):
+        """Get an IDP resource.
+
+        GET/HEAD /OS-FEDERATION/identity_providers/{idp_id}
+        """
+        ENFORCER.enforce_call(action='identity:get_identity_provider')
+        ref = PROVIDERS.federation_api.get_idp(idp_id)
+        return self.wrap_member(ref)
+
+    @validation.request_body_schema(
+        schema.identity_provider_create_request_body
+    )
+    @validation.response_body_schema(schema.identity_provider_response_body)
     def put(self, idp_id):
         """Create an idp resource for federated authentication.
 
@@ -132,21 +175,20 @@ class IdentityProvidersResource(_ResourceBase):
         """
         ENFORCER.enforce_call(action='identity:create_identity_provider')
         idp = self.request_body_json.get('identity_provider', {})
-        validation.lazy_validate(schema.identity_provider_create,
-                                 idp)
         idp = self._normalize_dict(idp)
         idp.setdefault('enabled', False)
-        idp_ref = PROVIDERS.federation_api.create_idp(
-            idp_id, idp)
+        idp_ref = PROVIDERS.federation_api.create_idp(idp_id, idp)
         return self.wrap_member(idp_ref), http.client.CREATED
 
+    @validation.request_body_schema(
+        schema.identity_provider_update_request_body
+    )
+    @validation.response_body_schema(schema.identity_provider_response_body)
     def patch(self, idp_id):
         ENFORCER.enforce_call(action='identity:update_identity_provider')
         idp = self.request_body_json.get('identity_provider', {})
-        validation.lazy_validate(schema.identity_provider_update, idp)
         idp = self._normalize_dict(idp)
-        idp_ref = PROVIDERS.federation_api.update_idp(
-            idp_id, idp)
+        idp_ref = PROVIDERS.federation_api.update_idp(idp_id, idp)
         return self.wrap_member(idp_ref)
 
     def delete(self, idp_id):
@@ -159,8 +201,7 @@ class _IdentityProvidersProtocolsResourceBase(_ResourceBase):
     collection_key = 'protocols'
     member_key = 'protocol'
     _public_parameters = frozenset(['id', 'mapping_id', 'links'])
-    json_home_additional_parameters = {
-        'idp_id': IDP_ID_PARAMETER_RELATION}
+    json_home_additional_parameters = {'idp_id': IDP_ID_PARAMETER_RELATION}
     json_home_collection_resource_name_override = 'identity_provider_protocols'
     json_home_member_resource_name_override = 'identity_provider_protocol'
 
@@ -176,11 +217,11 @@ class _IdentityProvidersProtocolsResourceBase(_ResourceBase):
         """
         ref.setdefault('links', {})
         ref['links']['identity_provider'] = ks_flask.base_url(
-            path=ref['idp_id'])
+            path=ref['idp_id']
+        )
 
 
 class IDPProtocolsListResource(_IdentityProvidersProtocolsResourceBase):
-
     def get(self, idp_id):
         """List protocols for an IDP.
 
@@ -197,7 +238,6 @@ class IDPProtocolsListResource(_IdentityProvidersProtocolsResourceBase):
 
 
 class IDPProtocolsCRUDResource(_IdentityProvidersProtocolsResourceBase):
-
     def get(self, idp_id, protocol_id):
         """Get protocols for an IDP.
 
@@ -215,10 +255,11 @@ class IDPProtocolsCRUDResource(_IdentityProvidersProtocolsResourceBase):
         """
         ENFORCER.enforce_call(action='identity:create_protocol')
         protocol = self.request_body_json.get('protocol', {})
-        validation.lazy_validate(schema.protocol_create, protocol)
+        ks_validation.lazy_validate(schema.protocol_create, protocol)
         protocol = self._normalize_dict(protocol)
-        ref = PROVIDERS.federation_api.create_protocol(idp_id, protocol_id,
-                                                       protocol)
+        ref = PROVIDERS.federation_api.create_protocol(
+            idp_id, protocol_id, protocol
+        )
         return self.wrap_member(ref), http.client.CREATED
 
     def patch(self, idp_id, protocol_id):
@@ -229,9 +270,10 @@ class IDPProtocolsCRUDResource(_IdentityProvidersProtocolsResourceBase):
         """
         ENFORCER.enforce_call(action='identity:update_protocol')
         protocol = self.request_body_json.get('protocol', {})
-        validation.lazy_validate(schema.protocol_update, protocol)
-        ref = PROVIDERS.federation_api.update_protocol(idp_id, protocol_id,
-                                                       protocol)
+        ks_validation.lazy_validate(schema.protocol_update, protocol)
+        ref = PROVIDERS.federation_api.update_protocol(
+            idp_id, protocol_id, protocol
+        )
         return self.wrap_member(ref)
 
     def delete(self, idp_id, protocol_id):
@@ -261,8 +303,9 @@ class MappingResource(_ResourceBase):
         HEAD/GET /OS-FEDERATION/mappings/{mapping_id}
         """
         ENFORCER.enforce_call(action='identity:get_mapping')
-        return self.wrap_member(PROVIDERS.federation_api.get_mapping(
-            mapping_id))
+        return self.wrap_member(
+            PROVIDERS.federation_api.get_mapping(mapping_id)
+        )
 
     def _list_mappings(self):
         """List mappings.
@@ -272,30 +315,57 @@ class MappingResource(_ResourceBase):
         ENFORCER.enforce_call(action='identity:list_mappings')
         return self.wrap_collection(PROVIDERS.federation_api.list_mappings())
 
+    def _internal_normalize_and_validate_attribute_mapping(
+        self, action_executed_message="created"
+    ):
+        mapping = self.request_body_json.get('mapping', {})
+        mapping = self._normalize_dict(mapping)
+
+        if not mapping.get('schema_version'):
+            default_schema_version = (
+                utils.get_default_attribute_mapping_schema_version()
+            )
+            LOG.debug(
+                "A mapping [%s] was %s without providing a "
+                "'schema_version'; therefore, we need to set one. The "
+                "current default is [%s]. We will use this value for "
+                "the attribute mapping being registered. It is "
+                "recommended that one does not rely on this default "
+                "value, as it can change, and the already persisted "
+                "attribute mappings will remain with the previous "
+                "default values.",
+                mapping,
+                action_executed_message,
+                default_schema_version,
+            )
+            mapping['schema_version'] = default_schema_version
+        utils.validate_mapping_structure(mapping)
+        return mapping
+
     def put(self, mapping_id):
         """Create a mapping.
 
         PUT /OS-FEDERATION/mappings/{mapping_id}
         """
         ENFORCER.enforce_call(action='identity:create_mapping')
-        mapping = self.request_body_json.get('mapping', {})
-        mapping = self._normalize_dict(mapping)
-        utils.validate_mapping_structure(mapping)
-        mapping_ref = PROVIDERS.federation_api.create_mapping(
-            mapping_id, mapping)
+
+        am = self._internal_normalize_and_validate_attribute_mapping(
+            "registered"
+        )
+        mapping_ref = PROVIDERS.federation_api.create_mapping(mapping_id, am)
+
         return self.wrap_member(mapping_ref), http.client.CREATED
 
     def patch(self, mapping_id):
-        """Update a mapping.
+        """Update an attribute mapping for identity federation.
 
         PATCH /OS-FEDERATION/mappings/{mapping_id}
         """
         ENFORCER.enforce_call(action='identity:update_mapping')
-        mapping = self.request_body_json.get('mapping', {})
-        mapping = self._normalize_dict(mapping)
-        utils.validate_mapping_structure(mapping)
-        mapping_ref = PROVIDERS.federation_api.update_mapping(
-            mapping_id, mapping)
+
+        am = self._internal_normalize_and_validate_attribute_mapping("updated")
+        mapping_ref = PROVIDERS.federation_api.update_mapping(mapping_id, am)
+
         return self.wrap_member(mapping_ref)
 
     def delete(self, mapping_id):
@@ -311,72 +381,109 @@ class MappingResource(_ResourceBase):
 class ServiceProvidersResource(_ResourceBase):
     collection_key = 'service_providers'
     member_key = 'service_provider'
-    _public_parameters = frozenset(['auth_url', 'id', 'enabled', 'description',
-                                    'links', 'relay_state_prefix', 'sp_url'])
-    _id_path_param_name_override = 'sp_id'
+    _public_parameters = frozenset(
+        [
+            'auth_url',
+            'id',
+            'enabled',
+            'description',
+            'links',
+            'relay_state_prefix',
+            'sp_url',
+        ]
+    )
     api_prefix = '/OS-FEDERATION'
 
-    def get(self, sp_id=None):
-        if sp_id is not None:
-            return self._get_service_provider(sp_id)
-        return self._list_service_providers()
-
-    def _get_service_provider(self, sp_id):
-        """Get a service provider.
-
-        GET/HEAD /OS-FEDERATION/service_providers/{sp_id}
-        """
-        ENFORCER.enforce_call(action='identity:get_service_provider')
-        return self.wrap_member(PROVIDERS.federation_api.get_sp(sp_id))
-
-    def _list_service_providers(self):
+    @validation.request_query_schema(
+        schema.service_provider_index_request_query
+    )
+    @validation.response_body_schema(
+        schema.service_provider_index_response_body
+    )
+    def get(self):
         """List service providers.
 
         GET/HEAD /OS-FEDERATION/service_providers
         """
         filters = ['id', 'enabled']
-        ENFORCER.enforce_call(action='identity:list_service_providers',
-                              filters=filters)
+        ENFORCER.enforce_call(
+            action='identity:list_service_providers', filters=filters
+        )
         hints = self.build_driver_hints(filters)
-        refs = [self.filter_params(r)
-                for r in
-                PROVIDERS.federation_api.list_sps(hints=hints)]
+        refs = [
+            self.filter_params(r)
+            for r in PROVIDERS.federation_api.list_sps(hints=hints)
+        ]
         return self.wrap_collection(refs, hints=hints)
 
-    def put(self, sp_id):
+
+class ServiceProviderResource(_ResourceBase):
+    collection_key = 'service_providers'
+    member_key = 'service_provider'
+    _public_parameters = frozenset(
+        [
+            'auth_url',
+            'id',
+            'enabled',
+            'description',
+            'links',
+            'relay_state_prefix',
+            'sp_url',
+        ]
+    )
+    api_prefix = '/OS-FEDERATION'
+
+    @validation.request_query_schema(None)
+    @validation.response_body_schema(schema.service_provider_response_body)
+    def get(self, service_provider_id):
+        """Get a service provider.
+
+        GET/HEAD /OS-FEDERATION/service_providers/{service_provider_id}
+        """
+        ENFORCER.enforce_call(action='identity:get_service_provider')
+        return self.wrap_member(
+            PROVIDERS.federation_api.get_sp(service_provider_id)
+        )
+
+    @validation.request_body_schema(
+        schema.service_provider_create_request_body
+    )
+    @validation.response_body_schema(schema.service_provider_response_body)
+    def put(self, service_provider_id):
         """Create a service provider.
 
-        PUT /OS-FEDERATION/service_providers/{sp_id}
+        PUT /OS-FEDERATION/service_providers/{service_provider_id}
         """
         ENFORCER.enforce_call(action='identity:create_service_provider')
         sp = self.request_body_json.get('service_provider', {})
-        validation.lazy_validate(schema.service_provider_create, sp)
         sp = self._normalize_dict(sp)
         sp.setdefault('enabled', False)
-        sp.setdefault('relay_state_prefix',
-                      CONF.saml.relay_state_prefix)
-        sp_ref = PROVIDERS.federation_api.create_sp(sp_id, sp)
+        sp.setdefault('relay_state_prefix', CONF.saml.relay_state_prefix)
+        sp_ref = PROVIDERS.federation_api.create_sp(service_provider_id, sp)
         return self.wrap_member(sp_ref), http.client.CREATED
 
-    def patch(self, sp_id):
+    @validation.request_body_schema(
+        schema.service_provider_update_request_body
+    )
+    @validation.response_body_schema(schema.service_provider_response_body)
+    def patch(self, service_provider_id):
         """Update a service provider.
 
-        PATCH /OS-FEDERATION/service_providers/{sp_id}
+        PATCH /OS-FEDERATION/service_providers/{service_provider_id}
         """
         ENFORCER.enforce_call(action='identity:update_service_provider')
         sp = self.request_body_json.get('service_provider', {})
-        validation.lazy_validate(schema.service_provider_update, sp)
         sp = self._normalize_dict(sp)
-        sp_ref = PROVIDERS.federation_api.update_sp(sp_id, sp)
+        sp_ref = PROVIDERS.federation_api.update_sp(service_provider_id, sp)
         return self.wrap_member(sp_ref)
 
-    def delete(self, sp_id):
+    def delete(self, service_provider_id):
         """Delete a service provider.
 
-        DELETE /OS-FEDERATION/service_providers/{sp_id}
+        DELETE /OS-FEDERATION/service_providers/{service_provider_id}
         """
         ENFORCER.enforce_call(action='identity:delete_service_provider')
-        PROVIDERS.federation_api.delete_sp(sp_id)
+        PROVIDERS.federation_api.delete_sp(service_provider_id)
         return None, http.client.NO_CONTENT
 
 
@@ -389,9 +496,9 @@ class SAML2MetadataResource(flask_restful.Resource):
         """
         metadata_path = CONF.saml.idp_metadata_path
         try:
-            with open(metadata_path, 'r') as metadata_handler:
+            with open(metadata_path) as metadata_handler:
                 metadata = metadata_handler.read()
-        except IOError as e:
+        except OSError as e:
             # Raise HTTP 500 in case Metadata file cannot be read.
             raise exception.MetadataFileError(reason=e)
         resp = flask.make_response(metadata, http.client.OK)
@@ -400,7 +507,6 @@ class SAML2MetadataResource(flask_restful.Resource):
 
 
 class OSFederationAuthResource(flask_restful.Resource):
-
     @ks_flask.unenforced_api
     def get(self, idp_id, protocol_id):
         """Authenticate from dedicated uri endpoint.
@@ -430,7 +536,7 @@ class OSFederationAuthResource(flask_restful.Resource):
                 'methods': [protocol_id],
                 protocol_id: {
                     'identity_provider': idp_id,
-                    'protocol': protocol_id
+                    'protocol': protocol_id,
                 },
             }
         }
@@ -454,17 +560,22 @@ class OSFederationAPI(ks_flask.APIBase):
             url='/saml2/metadata',
             resource_kwargs={},
             rel='metadata',
-            resource_relation_func=_build_resource_relation),
+            resource_relation_func=_build_resource_relation,
+        ),
         ks_flask.construct_resource_map(
             resource=OSFederationAuthResource,
-            url=('/identity_providers/<string:idp_id>/protocols/'
-                 '<string:protocol_id>/auth'),
+            url=(
+                '/identity_providers/<string:idp_id>/protocols/'
+                '<string:protocol_id>/auth'
+            ),
             resource_kwargs={},
             rel='identity_provider_protocol_auth',
             resource_relation_func=_build_resource_relation,
             path_vars={
                 'idp_id': IDP_ID_PARAMETER_RELATION,
-                'protocol_id': PROTOCOL_ID_PARAMETER_RELATION}),
+                'protocol_id': PROTOCOL_ID_PARAMETER_RELATION,
+            },
+        ),
     ]
 
 
@@ -472,8 +583,25 @@ class OSFederationIdentityProvidersAPI(ks_flask.APIBase):
     _name = 'identity_providers'
     _import_name = __name__
     _api_url_prefix = '/OS-FEDERATION'
-    resources = [IdentityProvidersResource]
-    resource_mapping = []
+    resources = []
+    resource_mapping = [
+        ks_flask.construct_resource_map(
+            resource=IdentityProvidersResource,
+            url='/identity_providers',
+            resource_kwargs={},
+            rel="identity_providers",
+            resource_relation_func=_build_resource_relation,
+            path_vars=None,
+        ),
+        ks_flask.construct_resource_map(
+            resource=IdentityProviderResource,
+            url='/identity_providers/<string:idp_id>',
+            resource_kwargs={},
+            rel="identity_provider",
+            resource_relation_func=_build_resource_relation,
+            path_vars={'idp_id': IDP_ID_PARAMETER_RELATION},
+        ),
+    ]
 
 
 class OSFederationIdentityProvidersProtocolsAPI(ks_flask.APIBase):
@@ -483,15 +611,17 @@ class OSFederationIdentityProvidersProtocolsAPI(ks_flask.APIBase):
     resource_mapping = [
         ks_flask.construct_resource_map(
             resource=IDPProtocolsCRUDResource,
-            url=('/OS-FEDERATION/identity_providers/<string:idp_id>/protocols/'
-                 '<string:protocol_id>'),
+            url=(
+                '/OS-FEDERATION/identity_providers/<string:idp_id>/protocols/'
+                '<string:protocol_id>'
+            ),
             resource_kwargs={},
             rel='identity_provider_protocol',
             resource_relation_func=_build_resource_relation,
             path_vars={
                 'idp_id': IDP_ID_PARAMETER_RELATION,
-                'protocol_id': PROTOCOL_ID_PARAMETER_RELATION
-            }
+                'protocol_id': PROTOCOL_ID_PARAMETER_RELATION,
+            },
         ),
         ks_flask.construct_resource_map(
             resource=IDPProtocolsListResource,
@@ -499,9 +629,7 @@ class OSFederationIdentityProvidersProtocolsAPI(ks_flask.APIBase):
             resource_kwargs={},
             rel='identity_provider_protocols',
             resource_relation_func=_build_resource_relation,
-            path_vars={
-                'idp_id': IDP_ID_PARAMETER_RELATION
-            }
+            path_vars={'idp_id': IDP_ID_PARAMETER_RELATION},
         ),
     ]
 
@@ -518,8 +646,26 @@ class OSFederationServiceProvidersAPI(ks_flask.APIBase):
     _name = 'service_providers'
     _import_name = __name__
     _api_url_prefix = '/OS-FEDERATION'
-    resources = [ServiceProvidersResource]
-    resource_mapping = []
+    resource_mapping = [
+        ks_flask.construct_resource_map(
+            resource=ServiceProvidersResource,
+            url='/service_providers',
+            resource_kwargs={},
+            rel="service_providers",
+            resource_relation_func=_build_resource_relation,
+            path_vars=None,
+        ),
+        ks_flask.construct_resource_map(
+            resource=ServiceProviderResource,
+            url='/service_providers/<string:service_provider_id>',
+            resource_kwargs={},
+            rel="service_provider",
+            resource_relation_func=_build_resource_relation,
+            path_vars={
+                'service_provider_id': SERVICE_PROVIDER_ID_PARAMETER_RELATION
+            },
+        ),
+    ]
 
 
 APIs = (
@@ -527,5 +673,5 @@ APIs = (
     OSFederationIdentityProvidersAPI,
     OSFederationIdentityProvidersProtocolsAPI,
     OSFederationMappingsAPI,
-    OSFederationServiceProvidersAPI
+    OSFederationServiceProvidersAPI,
 )
