@@ -12,4 +12,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystone.auth import controllers  # noqa
+# NOTE(notmorgan): Be careful in adjusting whitespace here, flake8 checks
+# get cranky.
+from keystone.auth import core  # noqa
