@@ -13,8 +13,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import itertools
-
 import sqlalchemy
 from sqlalchemy.sql import true
 
@@ -26,11 +24,10 @@ import keystone.conf
 from keystone import exception
 from keystone.i18n import _
 
-
 CONF = keystone.conf.CONF
 
 
-class Region(sql.ModelBase, sql.DictBase):
+class Region(sql.ModelBase, sql.ModelDictMixinWithExtras):
     __tablename__ = 'region'
     attributes = ['id', 'description', 'parent_region_id']
     id = sql.Column(sql.String(255), primary_key=True)
@@ -49,39 +46,63 @@ class Region(sql.ModelBase, sql.DictBase):
     endpoints = sqlalchemy.orm.relationship("Endpoint", backref="region")
 
 
-class Service(sql.ModelBase, sql.DictBase):
+class Service(sql.ModelBase, sql.ModelDictMixinWithExtras):
     __tablename__ = 'service'
     attributes = ['id', 'type', 'enabled']
     id = sql.Column(sql.String(64), primary_key=True)
     type = sql.Column(sql.String(255))
-    enabled = sql.Column(sql.Boolean, nullable=False, default=True,
-                         server_default=sqlalchemy.sql.expression.true())
+    enabled = sql.Column(
+        sql.Boolean,
+        nullable=False,
+        default=True,
+        server_default=sqlalchemy.sql.expression.true(),
+    )
     extra = sql.Column(sql.JsonBlob())
     endpoints = sqlalchemy.orm.relationship("Endpoint", backref="service")
 
 
-class Endpoint(sql.ModelBase, sql.DictBase):
+class Endpoint(sql.ModelBase, sql.ModelDictMixinWithExtras):
     __tablename__ = 'endpoint'
-    attributes = ['id', 'interface', 'region_id', 'service_id', 'url',
-                  'legacy_endpoint_id', 'enabled']
+    attributes = [
+        'id',
+        'interface',
+        'region_id',
+        'service_id',
+        'url',
+        'legacy_endpoint_id',
+        'enabled',
+    ]
     id = sql.Column(sql.String(64), primary_key=True)
     legacy_endpoint_id = sql.Column(sql.String(64))
     interface = sql.Column(sql.String(8), nullable=False)
-    region_id = sql.Column(sql.String(255),
-                           sql.ForeignKey('region.id',
-                                          ondelete='RESTRICT'),
-                           nullable=True,
-                           default=None)
-    service_id = sql.Column(sql.String(64),
-                            sql.ForeignKey('service.id'),
-                            nullable=False)
+    region_id = sql.Column(
+        sql.String(255),
+        sql.ForeignKey('region.id', ondelete='RESTRICT'),
+        nullable=True,
+        default=None,
+    )
+    service_id = sql.Column(
+        sql.String(64), sql.ForeignKey('service.id'), nullable=False
+    )
     url = sql.Column(sql.Text(), nullable=False)
-    enabled = sql.Column(sql.Boolean, nullable=False, default=True,
-                         server_default=sqlalchemy.sql.expression.true())
+    enabled = sql.Column(
+        sql.Boolean,
+        nullable=False,
+        default=True,
+        server_default=sqlalchemy.sql.expression.true(),
+    )
     extra = sql.Column(sql.JsonBlob())
 
+    @classmethod
+    def from_dict(cls, endpoint_dict):
+        """Override from_dict to set enabled if missing."""
+        new_dict = endpoint_dict.copy()
+        if new_dict.get('enabled') is None:
+            new_dict['enabled'] = True
+        return super().from_dict(new_dict)
 
-class Catalog(base.CatalogDriverV8):
+
+class Catalog(base.CatalogDriverBase):
     # Regions
     def list_regions(self, hints):
         with sql.session_for_read() as session:
@@ -90,7 +111,7 @@ class Catalog(base.CatalogDriverV8):
             return [s.to_dict() for s in list(regions)]
 
     def _get_region(self, session, region_id):
-        ref = session.query(Region).get(region_id)
+        ref = session.get(Region, region_id)
         if not ref:
             raise exception.RegionNotFound(region_id=region_id)
         return ref
@@ -166,6 +187,7 @@ class Catalog(base.CatalogDriverV8):
             for attr in Region.attributes:
                 if attr != 'id':
                     setattr(ref, attr, getattr(new_region, attr))
+            ref.extra = new_region.extra
             return ref.to_dict()
 
     # Services
@@ -177,7 +199,7 @@ class Catalog(base.CatalogDriverV8):
             return [s.to_dict() for s in list(services)]
 
     def _get_service(self, session, service_id):
-        ref = session.query(Service).get(service_id)
+        ref = session.get(Service, service_id)
         if not ref:
             raise exception.ServiceNotFound(service_id=service_id)
         return ref
@@ -211,11 +233,11 @@ class Catalog(base.CatalogDriverV8):
             return ref.to_dict()
 
     # Endpoints
-    def create_endpoint(self, endpoint_id, endpoint_ref):
-        new_endpoint = Endpoint.from_dict(endpoint_ref)
+    def create_endpoint(self, endpoint_id, endpoint):
         with sql.session_for_write() as session:
-            session.add(new_endpoint)
-        return new_endpoint.to_dict()
+            endpoint_ref = Endpoint.from_dict(endpoint)
+            session.add(endpoint_ref)
+            return endpoint_ref.to_dict()
 
     def delete_endpoint(self, endpoint_id):
         with sql.session_for_write() as session:
@@ -251,37 +273,38 @@ class Catalog(base.CatalogDriverV8):
             ref.extra = new_endpoint.extra
             return ref.to_dict()
 
-    def get_catalog(self, user_id, tenant_id):
+    def get_catalog(self, user_id, project_id):
         """Retrieve and format the V2 service catalog.
 
         :param user_id: The id of the user who has been authenticated for
             creating service catalog.
-        :param tenant_id: The id of the project. 'tenant_id' will be None
+        :param project_id: The id of the project. 'project_id' will be None
             in the case this being called to create a catalog to go in a
             domain scoped token. In this case, any endpoint that requires
-            a tenant_id as part of their URL will be skipped (as would a whole
+            a project_id as part of their URL will be skipped (as would a whole
             service if, as a consequence, it has no valid endpoints).
 
         :returns: A nested dict representing the service catalog or an
                   empty dict.
 
         """
-        substitutions = dict(
-            itertools.chain(CONF.items(), CONF.eventlet_server.items()))
+        substitutions = dict(CONF.items())
         substitutions.update({'user_id': user_id})
         silent_keyerror_failures = []
-        if tenant_id:
-            substitutions.update({
-                'tenant_id': tenant_id,
-                'project_id': tenant_id
-            })
+        if project_id:
+            substitutions.update(
+                {'tenant_id': project_id, 'project_id': project_id}
+            )
         else:
-            silent_keyerror_failures = ['tenant_id', 'project_id', ]
+            silent_keyerror_failures = ['tenant_id', 'project_id']
 
         with sql.session_for_read() as session:
-            endpoints = (session.query(Endpoint).
-                         options(sql.joinedload(Endpoint.service)).
-                         filter(Endpoint.enabled == true()).all())
+            endpoints = (
+                session.query(Endpoint)
+                .options(sql.joinedload(Endpoint.service))
+                .filter(Endpoint.enabled == true())
+                .all()
+            )
 
             catalog = {}
 
@@ -290,8 +313,10 @@ class Catalog(base.CatalogDriverV8):
                     continue
                 try:
                     formatted_url = utils.format_url(
-                        endpoint['url'], substitutions,
-                        silent_keyerror_failures=silent_keyerror_failures)
+                        endpoint['url'],
+                        substitutions,
+                        silent_keyerror_failures=silent_keyerror_failures,
+                    )
                     if formatted_url is not None:
                         url = formatted_url
                     else:
@@ -304,56 +329,58 @@ class Catalog(base.CatalogDriverV8):
                 default_service = {
                     'id': endpoint['id'],
                     'name': endpoint.service.extra.get('name', ''),
-                    'publicURL': ''
+                    'publicURL': '',
                 }
                 catalog.setdefault(region, {})
                 catalog[region].setdefault(service_type, default_service)
-                interface_url = '%sURL' % endpoint['interface']
+                interface_url = '{}URL'.format(endpoint['interface'])
                 catalog[region][service_type][interface_url] = url
 
             return catalog
 
-    def get_v3_catalog(self, user_id, tenant_id):
+    def get_v3_catalog(self, user_id, project_id):
         """Retrieve and format the current V3 service catalog.
 
         :param user_id: The id of the user who has been authenticated for
             creating service catalog.
-        :param tenant_id: The id of the project. 'tenant_id' will be None in
+        :param project_id: The id of the project. 'project_id' will be None in
             the case this being called to create a catalog to go in a domain
             scoped token. In this case, any endpoint that requires a
-            tenant_id as part of their URL will be skipped.
+            project_id as part of their URL will be skipped.
 
         :returns: A list representing the service catalog or an empty list
 
         """
-        d = dict(
-            itertools.chain(CONF.items(), CONF.eventlet_server.items()))
+        d = dict(CONF.items())
         d.update({'user_id': user_id})
         silent_keyerror_failures = []
-        if tenant_id:
-            d.update({
-                'tenant_id': tenant_id,
-                'project_id': tenant_id,
-            })
+        if project_id:
+            d.update({'tenant_id': project_id, 'project_id': project_id})
         else:
-            silent_keyerror_failures = ['tenant_id', 'project_id', ]
+            silent_keyerror_failures = ['tenant_id', 'project_id']
 
         with sql.session_for_read() as session:
-            services = (session.query(Service).filter(
-                Service.enabled == true()).options(
-                    sql.joinedload(Service.endpoints)).all())
+            services = (
+                session.query(Service)
+                .filter(Service.enabled == true())
+                .options(sql.joinedload(Service.endpoints))
+                .all()
+            )
 
             def make_v3_endpoints(endpoints):
-                for endpoint in (ep.to_dict()
-                                 for ep in endpoints if ep.enabled):
+                for endpoint in (
+                    ep.to_dict() for ep in endpoints if ep.enabled
+                ):
                     del endpoint['service_id']
                     del endpoint['legacy_endpoint_id']
                     del endpoint['enabled']
                     endpoint['region'] = endpoint['region_id']
                     try:
                         formatted_url = utils.format_url(
-                            endpoint['url'], d,
-                            silent_keyerror_failures=silent_keyerror_failures)
+                            endpoint['url'],
+                            d,
+                            silent_keyerror_failures=silent_keyerror_failures,
+                        )
                         if formatted_url:
                             endpoint['url'] = formatted_url
                         else:
@@ -373,22 +400,71 @@ class Catalog(base.CatalogDriverV8):
                 service['name'] = svc.extra.get('name', '')
                 return service
 
-            return [make_v3_service(svc) for svc in services]
+            # Build the unfiltered catalog, this is the catalog that is
+            # returned if endpoint filtering is not performed and the
+            # option of `return_all_endpoints_if_no_filter` is set to true.
+            catalog_ref = [make_v3_service(svc) for svc in services]
+
+            # Filter the `catalog_ref` above by any project-endpoint
+            # association configured by endpoint filter.
+            filtered_endpoints = {}
+            if project_id:
+                filtered_endpoints = (
+                    self.catalog_api.list_endpoints_for_project(project_id)
+                )
+            # endpoint filter is enabled, only return the filtered endpoints.
+            if filtered_endpoints:
+                filtered_ids = list(filtered_endpoints.keys())
+                # This is actually working on the copy of `catalog_ref` since
+                # the index will be shifted if remove/add any entry for the
+                # original one.
+                for service in catalog_ref[:]:
+                    endpoints = service['endpoints']
+                    for endpoint in endpoints[:]:
+                        endpoint_id = endpoint['id']
+                        # remove the endpoint that is not associated with
+                        # the project.
+                        if endpoint_id not in filtered_ids:
+                            service['endpoints'].remove(endpoint)
+                            continue
+                        # remove the disabled endpoint from the list.
+                        if not filtered_endpoints[endpoint_id]['enabled']:
+                            service['endpoints'].remove(endpoint)
+                    # NOTE(davechen): The service will not be included in the
+                    # catalog if the service doesn't have any endpoint when
+                    # endpoint filter is enabled, this is inconsistent with
+                    # full catalog that is returned when endpoint filter is
+                    # disabled.
+                    if not service.get('endpoints'):
+                        catalog_ref.remove(service)
+            # When it arrives here it means it's domain scoped token (
+            # `project_id` is not set) or it's a project scoped token
+            # but the endpoint filtering is not performed.
+            # Both of them tell us the endpoint filtering is not enabled, so
+            # check the option of `return_all_endpoints_if_no_filter`, it will
+            # judge whether a full unfiltered catalog or a empty service
+            # catalog will be returned.
+            elif not CONF.endpoint_filter.return_all_endpoints_if_no_filter:
+                return []
+            return catalog_ref
 
     @sql.handle_conflicts(conflict_type='project_endpoint')
     def add_endpoint_to_project(self, endpoint_id, project_id):
         with sql.session_for_write() as session:
-            endpoint_filter_ref = ProjectEndpoint(endpoint_id=endpoint_id,
-                                                  project_id=project_id)
+            endpoint_filter_ref = ProjectEndpoint(
+                endpoint_id=endpoint_id, project_id=project_id
+            )
             session.add(endpoint_filter_ref)
 
     def _get_project_endpoint_ref(self, session, endpoint_id, project_id):
-        endpoint_filter_ref = session.query(ProjectEndpoint).get(
-            (endpoint_id, project_id))
+        endpoint_filter_ref = session.get(
+            ProjectEndpoint, (endpoint_id, project_id)
+        )
         if endpoint_filter_ref is None:
-            msg = _('Endpoint %(endpoint_id)s not found in project '
-                    '%(project_id)s') % {'endpoint_id': endpoint_id,
-                                         'project_id': project_id}
+            msg = _(
+                'Endpoint %(endpoint_id)s not found in project '
+                '%(project_id)s'
+            ) % {'endpoint_id': endpoint_id, 'project_id': project_id}
             raise exception.NotFound(msg)
         return endpoint_filter_ref
 
@@ -399,7 +475,8 @@ class Catalog(base.CatalogDriverV8):
     def remove_endpoint_from_project(self, endpoint_id, project_id):
         with sql.session_for_write() as session:
             endpoint_filter_ref = self._get_project_endpoint_ref(
-                session, endpoint_id, project_id)
+                session, endpoint_id, project_id
+            )
             session.delete(endpoint_filter_ref)
 
     def list_endpoints_for_project(self, project_id):
@@ -435,44 +512,49 @@ class Catalog(base.CatalogDriverV8):
             return endpoint_group_ref.to_dict()
 
     def _get_endpoint_group(self, session, endpoint_group_id):
-        endpoint_group_ref = session.query(EndpointGroup).get(
-            endpoint_group_id)
+        endpoint_group_ref = session.get(EndpointGroup, endpoint_group_id)
         if endpoint_group_ref is None:
             raise exception.EndpointGroupNotFound(
-                endpoint_group_id=endpoint_group_id)
+                endpoint_group_id=endpoint_group_id
+            )
         return endpoint_group_ref
 
     def get_endpoint_group(self, endpoint_group_id):
         with sql.session_for_read() as session:
-            endpoint_group_ref = self._get_endpoint_group(session,
-                                                          endpoint_group_id)
+            endpoint_group_ref = self._get_endpoint_group(
+                session, endpoint_group_id
+            )
             return endpoint_group_ref.to_dict()
 
     def update_endpoint_group(self, endpoint_group_id, endpoint_group):
         with sql.session_for_write() as session:
-            endpoint_group_ref = self._get_endpoint_group(session,
-                                                          endpoint_group_id)
+            endpoint_group_ref = self._get_endpoint_group(
+                session, endpoint_group_id
+            )
             old_endpoint_group = endpoint_group_ref.to_dict()
             old_endpoint_group.update(endpoint_group)
             new_endpoint_group = EndpointGroup.from_dict(old_endpoint_group)
             for attr in EndpointGroup.mutable_attributes:
-                setattr(endpoint_group_ref, attr,
-                        getattr(new_endpoint_group, attr))
+                setattr(
+                    endpoint_group_ref, attr, getattr(new_endpoint_group, attr)
+                )
             return endpoint_group_ref.to_dict()
 
     def delete_endpoint_group(self, endpoint_group_id):
         with sql.session_for_write() as session:
-            endpoint_group_ref = self._get_endpoint_group(session,
-                                                          endpoint_group_id)
+            endpoint_group_ref = self._get_endpoint_group(
+                session, endpoint_group_id
+            )
             self._delete_endpoint_group_association_by_endpoint_group(
-                session, endpoint_group_id)
+                session, endpoint_group_id
+            )
             session.delete(endpoint_group_ref)
 
     def get_endpoint_group_in_project(self, endpoint_group_id, project_id):
         with sql.session_for_read() as session:
-            ref = self._get_endpoint_group_in_project(session,
-                                                      endpoint_group_id,
-                                                      project_id)
+            ref = self._get_endpoint_group_in_project(
+                session, endpoint_group_id, project_id
+            )
             return ref.to_dict()
 
     @sql.handle_conflicts(conflict_type='project_endpoint_group')
@@ -480,24 +562,28 @@ class Catalog(base.CatalogDriverV8):
         with sql.session_for_write() as session:
             # Create a new Project Endpoint group entity
             endpoint_group_project_ref = ProjectEndpointGroupMembership(
-                endpoint_group_id=endpoint_group_id, project_id=project_id)
+                endpoint_group_id=endpoint_group_id, project_id=project_id
+            )
             session.add(endpoint_group_project_ref)
 
-    def _get_endpoint_group_in_project(self, session,
-                                       endpoint_group_id, project_id):
-        endpoint_group_project_ref = session.query(
-            ProjectEndpointGroupMembership).get((endpoint_group_id,
-                                                 project_id))
+    def _get_endpoint_group_in_project(
+        self, session, endpoint_group_id, project_id
+    ):
+        endpoint_group_project_ref = session.get(
+            ProjectEndpointGroupMembership, (endpoint_group_id, project_id)
+        )
         if endpoint_group_project_ref is None:
             msg = _('Endpoint Group Project Association not found')
             raise exception.NotFound(msg)
         else:
             return endpoint_group_project_ref
 
-    def list_endpoint_groups(self):
+    def list_endpoint_groups(self, hints):
         with sql.session_for_read() as session:
             query = session.query(EndpointGroup)
-            endpoint_group_refs = query.all()
+            endpoint_group_refs = sql.filter_limit_query(
+                EndpointGroup, query, hints
+            )
             return [e.to_dict() for e in endpoint_group_refs]
 
     def list_endpoint_groups_for_project(self, project_id):
@@ -507,11 +593,13 @@ class Catalog(base.CatalogDriverV8):
             endpoint_group_refs = query.all()
             return [ref.to_dict() for ref in endpoint_group_refs]
 
-    def remove_endpoint_group_from_project(self, endpoint_group_id,
-                                           project_id):
+    def remove_endpoint_group_from_project(
+        self, endpoint_group_id, project_id
+    ):
         with sql.session_for_write() as session:
             endpoint_group_project_ref = self._get_endpoint_group_in_project(
-                session, endpoint_group_id, project_id)
+                session, endpoint_group_id, project_id
+            )
             session.delete(endpoint_group_project_ref)
 
     def list_projects_associated_with_endpoint_group(self, endpoint_group_id):
@@ -522,7 +610,8 @@ class Catalog(base.CatalogDriverV8):
             return [ref.to_dict() for ref in endpoint_group_refs]
 
     def _delete_endpoint_group_association_by_endpoint_group(
-            self, session, endpoint_group_id):
+        self, session, endpoint_group_id
+    ):
         query = session.query(ProjectEndpointGroupMembership)
         query = query.filter_by(endpoint_group_id=endpoint_group_id)
         query.delete()
@@ -539,12 +628,8 @@ class ProjectEndpoint(sql.ModelBase, sql.ModelDictMixin):
 
     __tablename__ = 'project_endpoint'
     attributes = ['endpoint_id', 'project_id']
-    endpoint_id = sql.Column(sql.String(64),
-                             primary_key=True,
-                             nullable=False)
-    project_id = sql.Column(sql.String(64),
-                            primary_key=True,
-                            nullable=False)
+    endpoint_id = sql.Column(sql.String(64), primary_key=True, nullable=False)
+    project_id = sql.Column(sql.String(64), primary_key=True, nullable=False)
 
 
 class EndpointGroup(sql.ModelBase, sql.ModelDictMixin):
@@ -564,9 +649,10 @@ class ProjectEndpointGroupMembership(sql.ModelBase, sql.ModelDictMixin):
 
     __tablename__ = 'project_endpoint_group'
     attributes = ['endpoint_group_id', 'project_id']
-    endpoint_group_id = sql.Column(sql.String(64),
-                                   sql.ForeignKey('endpoint_group.id'),
-                                   nullable=False)
+    endpoint_group_id = sql.Column(
+        sql.String(64), sql.ForeignKey('endpoint_group.id'), nullable=False
+    )
     project_id = sql.Column(sql.String(64), nullable=False)
-    __table_args__ = (sql.PrimaryKeyConstraint('endpoint_group_id',
-                                               'project_id'),)
+    __table_args__ = (
+        sql.PrimaryKeyConstraint('endpoint_group_id', 'project_id'),
+    )
