@@ -23,25 +23,25 @@ deprecated_ec2_get_credential = policy.DeprecatedRule(
     name=base.IDENTITY % 'ec2_get_credential',
     check_str=base.RULE_ADMIN_OR_CREDENTIAL_OWNER,
     deprecated_reason=DEPRECATED_REASON,
-    deprecated_since=versionutils.deprecated.TRAIN
+    deprecated_since=versionutils.deprecated.TRAIN,
 )
 deprecated_ec2_list_credentials = policy.DeprecatedRule(
     name=base.IDENTITY % 'ec2_list_credentials',
     check_str=base.RULE_ADMIN_OR_OWNER,
     deprecated_reason=DEPRECATED_REASON,
-    deprecated_since=versionutils.deprecated.TRAIN
+    deprecated_since=versionutils.deprecated.TRAIN,
 )
 deprecated_ec2_create_credential = policy.DeprecatedRule(
     name=base.IDENTITY % 'ec2_create_credential',
     check_str=base.RULE_ADMIN_OR_OWNER,
     deprecated_reason=DEPRECATED_REASON,
-    deprecated_since=versionutils.deprecated.TRAIN
+    deprecated_since=versionutils.deprecated.TRAIN,
 )
 deprecated_ec2_delete_credential = policy.DeprecatedRule(
     name=base.IDENTITY % 'ec2_delete_credential',
     check_str=base.RULE_ADMIN_OR_CREDENTIAL_OWNER,
     deprecated_reason=DEPRECATED_REASON,
-    deprecated_since=versionutils.deprecated.TRAIN
+    deprecated_since=versionutils.deprecated.TRAIN,
 )
 
 
@@ -51,18 +51,24 @@ ec2_credential_policies = [
         check_str=base.ADMIN_OR_SYSTEM_READER_OR_CRED_OWNER,
         scope_types=['system', 'project'],
         description='Show ec2 credential details.',
-        operations=[{'path': ('/v3/users/{user_id}/credentials/OS-EC2/'
-                              '{credential_id}'),
-                     'method': 'GET'}],
-        deprecated_rule=deprecated_ec2_get_credential
+        operations=[
+            {
+                'path': (
+                    '/v3/users/{user_id}/credentials/OS-EC2/{credential_id}'
+                ),
+                'method': 'GET',
+            }
+        ],
+        deprecated_rule=deprecated_ec2_get_credential,
     ),
     policy.DocumentedRuleDefault(
         name=base.IDENTITY % 'ec2_list_credentials',
         check_str=base.ADMIN_OR_SYSTEM_READER_OR_OWNER,
         scope_types=['system', 'project'],
         description='List ec2 credentials.',
-        operations=[{'path': '/v3/users/{user_id}/credentials/OS-EC2',
-                     'method': 'GET'}],
+        operations=[
+            {'path': '/v3/users/{user_id}/credentials/OS-EC2', 'method': 'GET'}
+        ],
         deprecated_rule=deprecated_ec2_list_credentials,
     ),
     policy.DocumentedRuleDefault(
@@ -70,8 +76,12 @@ ec2_credential_policies = [
         check_str=base.RULE_ADMIN_OR_OWNER,
         scope_types=['system', 'project'],
         description='Create ec2 credential.',
-        operations=[{'path': '/v3/users/{user_id}/credentials/OS-EC2',
-                     'method': 'POST'}],
+        operations=[
+            {
+                'path': '/v3/users/{user_id}/credentials/OS-EC2',
+                'method': 'POST',
+            }
+        ],
         deprecated_rule=deprecated_ec2_create_credential,
     ),
     policy.DocumentedRuleDefault(
@@ -79,11 +89,16 @@ ec2_credential_policies = [
         check_str=base.ADMIN_OR_CRED_OWNER,
         scope_types=['system', 'project'],
         description='Delete ec2 credential.',
-        operations=[{'path': ('/v3/users/{user_id}/credentials/OS-EC2/'
-                              '{credential_id}'),
-                     'method': 'DELETE'}],
+        operations=[
+            {
+                'path': (
+                    '/v3/users/{user_id}/credentials/OS-EC2/{credential_id}'
+                ),
+                'method': 'DELETE',
+            }
+        ],
         deprecated_rule=deprecated_ec2_delete_credential,
-    )
+    ),
 ]
 
 
