@@ -18,6 +18,7 @@ Before using this module, call initialize(). This has to be done before
 CONF() because it sets up configuration options.
 
 """
+
 import datetime
 import functools
 
@@ -117,7 +118,6 @@ ModelBase.__init__ = initialize_decorator(ModelBase.__init__)
 
 # Special Fields
 class JsonBlob(sql_types.TypeDecorator):
-
     impl = sql.Text
     # NOTE(ralonsoh): set to True as any other TypeDecorator in SQLAlchemy
     # https://docs.sqlalchemy.org/en/14/core/custom_types.html# \
@@ -256,7 +256,6 @@ class ModelDictMixinWithExtras(models.ModelBase):
 
 
 class ModelDictMixin(models.ModelBase):
-
     @classmethod
     def from_dict(cls, d):
         """Return a model instance from a dictionary."""
@@ -413,13 +412,13 @@ def _filter(model, query, hints):
 
         if filter_['comparator'] == 'contains':
             _WontMatch.check(filter_['value'], column_attr)
-            query_term = column_attr.ilike('%%%s%%' % filter_['value'])
+            query_term = column_attr.ilike('%{}%'.format(filter_['value']))
         elif filter_['comparator'] == 'startswith':
             _WontMatch.check(filter_['value'], column_attr)
-            query_term = column_attr.ilike('%s%%' % filter_['value'])
+            query_term = column_attr.ilike('{}%'.format(filter_['value']))
         elif filter_['comparator'] == 'endswith':
             _WontMatch.check(filter_['value'], column_attr)
-            query_term = column_attr.ilike('%%%s' % filter_['value'])
+            query_term = column_attr.ilike('%{}'.format(filter_['value']))
         else:
             # It's a filter we don't understand, so let the caller
             # work out if they need to do something with it.
@@ -494,6 +493,40 @@ def _limit(query, hints):
     return query
 
 
+def filter_query(model, query, hints):
+    """Apply filtering to a query.
+
+    :param model: table model
+    :param query: query to apply filters to
+    :param hints: contains the list of filters and limit details.  This may
+                  be None, indicating that there are no filters or limits
+                  to be applied. If it's not None, then any filters
+                  satisfied here will be removed so that the caller will
+                  know if any filters remain.
+
+    :returns: query updated with any filters and limits satisfied
+
+    """
+    if hints is None:
+        return query
+
+    # First try and satisfy any filters
+    query = _filter(model, query, hints)
+
+    if hints.cannot_match:
+        # Nothing's going to match, so don't bother with the query.
+        return []
+
+    # NOTE(henry-nash): Any unsatisfied filters will have been left in
+    # the hints list for the controller to handle. We can only try and
+    # limit here if all the filters are already satisfied since, if not,
+    # doing so might mess up the final results. If there are still
+    # unsatisfied filters, we have to leave any limiting to the controller
+    # as well.
+
+    return query
+
+
 def filter_limit_query(model, query, hints):
     """Apply filtering and limit to a query.
 
