@@ -19,7 +19,6 @@ import os
 from alembic import command as alembic_api
 from alembic import config as alembic_config
 from alembic import migration as alembic_migration
-from alembic.runtime import environment as alembic_environment
 from alembic import script as alembic_script
 from oslo_db import exception as db_exception
 from oslo_log import log as logging
@@ -37,11 +36,7 @@ EXPAND_BRANCH = 'expand'
 DATA_MIGRATION_BRANCH = 'data_migration'
 CONTRACT_BRANCH = 'contract'
 
-RELEASES = (
-    'yoga',
-    'bobcat',
-    '2024.01',
-)
+RELEASES = ('yoga', 'bobcat', '2024.01')
 MILESTONES = (
     'yoga',
     # Do not add the milestone until the end of the release
@@ -49,9 +44,7 @@ MILESTONES = (
 CURRENT_RELEASE = RELEASES[-1]
 MIGRATION_BRANCHES = (EXPAND_BRANCH, CONTRACT_BRANCH)
 VERSIONS_PATH = os.path.join(
-    os.path.dirname(sql.__file__),
-    'migrations',
-    'versions',
+    os.path.dirname(sql.__file__), 'migrations', 'versions'
 )
 
 
@@ -78,8 +71,7 @@ def _find_alembic_conf():
     :returns: An instance of ``alembic.config.Config``
     """
     path = os.path.join(
-        os.path.abspath(os.path.dirname(__file__)),
-        'alembic.ini',
+        os.path.abspath(os.path.dirname(__file__)), 'alembic.ini'
     )
 
     config = alembic_config.Config(os.path.abspath(path))
@@ -113,11 +105,6 @@ def _get_current_heads(engine, config):
         context = alembic_migration.MigrationContext.configure(conn)
         heads = context.get_current_heads()
 
-    heads_map = _get_head_maps(heads, script)
-    return heads_map
-
-
-def _get_head_maps(heads, script):
     heads_map = {}
 
     for head in heads:
@@ -155,17 +142,6 @@ def get_current_heads():
     return heads
 
 
-def get_head_revisions():
-    """Get the available head for each the expand and contract branches."""
-    config = _find_alembic_conf()
-    script = alembic_script.ScriptDirectory.from_config(config)
-    context = alembic_environment.EnvironmentContext(config, script)
-    heads = context.get_head_revisions()
-    heads_map = _get_head_maps(heads, script)
-
-    return heads_map
-
-
 def _is_database_under_alembic_control(engine):
     with engine.connect() as conn:
         context = alembic_migration.MigrationContext.configure(conn)
