@@ -16,11 +16,11 @@ import re
 import jsonschema
 from oslo_config import cfg
 from oslo_log import log
-import six
+from oslo_utils import timeutils
+from oslo_utils import uuidutils
 
 from keystone import exception
-from keystone.i18n import _, _LE
-
+from keystone.i18n import _
 
 CONF = cfg.CONF
 LOG = log.getLogger(__name__)
@@ -30,29 +30,61 @@ LOG = log.getLogger(__name__)
 def validate_password(password):
     pattern = CONF.security_compliance.password_regex
     if pattern:
-        if not isinstance(password, six.string_types):
+        if not isinstance(password, str):
             detail = _("Password must be a string type")
             raise exception.PasswordValidationError(detail=detail)
         try:
             if not re.match(pattern, password):
                 pattern_desc = (
-                    CONF.security_compliance.password_regex_description)
-                detail = _("The password does not meet the requirements: "
-                           "%(message)s") % {'message': pattern_desc}
-                raise exception.PasswordValidationError(detail=detail)
+                    CONF.security_compliance.password_regex_description
+                )
+                raise exception.PasswordRequirementsValidationError(
+                    detail=pattern_desc
+                )
         except re.error:
-            msg = _LE("Unable to validate password due to invalid regular "
-                      "expression - password_regex: ")
+            msg = (
+                "Unable to validate password due to invalid regular "
+                "expression - password_regex: %s"
+            )
             LOG.error(msg, pattern)
-            detail = _("Unable to validate password due to invalid "
-                       "configuration")
+            detail = _(
+                "Unable to validate password due to invalid configuration"
+            )
             raise exception.PasswordValidationError(detail=detail)
 
 
-class SchemaValidator(object):
+_FORMAT_CHECKER = jsonschema.FormatChecker()
+
+
+@_FORMAT_CHECKER.checks("date-time")
+def _validate_datetime_format(instance: object) -> bool:
+    # format checks constrain to the relevant primitive type
+    # https://github.com/OAI/OpenAPI-Specification/issues/3148
+    if not isinstance(instance, str):
+        return True
+    try:
+        timeutils.parse_isotime(instance)
+    except ValueError:
+        return False
+    else:
+        return True
+
+
+@_FORMAT_CHECKER.checks("uuid")
+def _validate_uuid_format(instance: object) -> bool:
+    # format checks constrain to the relevant primitive type
+    # https://github.com/OAI/OpenAPI-Specification/issues/3148
+    if not isinstance(instance, str):
+        return True
+
+    return uuidutils.is_uuid_like(instance)
+
+
+class SchemaValidator:
     """Resource reference validator class."""
 
-    validator_org = jsonschema.Draft4Validator
+    # Use 2020 Schema consistently in all other OpenStack services
+    validator_org = jsonschema.Draft202012Validator
 
     def __init__(self, schema):
         # NOTE(lbragstad): If at some point in the future we want to extend
@@ -63,10 +95,10 @@ class SchemaValidator(object):
         # didn't by default. See the Nova V3 validator for details on how this
         # is done.
         validators = {}
-        validator_cls = jsonschema.validators.extend(self.validator_org,
-                                                     validators)
-        fc = jsonschema.FormatChecker()
-        self.validator = validator_cls(schema, format_checker=fc)
+        validator_cls = jsonschema.validators.extend(
+            self.validator_org, validators
+        )
+        self.validator = validator_cls(schema, format_checker=_FORMAT_CHECKER)
 
     def validate(self, *args, **kwargs):
         try:
@@ -82,9 +114,10 @@ class SchemaValidator(object):
                 # too long, then we should build the masking in here so that
                 # we don't expose sensitive user information in the event it
                 # fails validation.
-                detail = _("Invalid input for field '%(path)s': "
-                           "%(message)s") % {'path': ex.path.pop(),
-                                             'message': ex.message}
+                path = '/'.join(map(str, ex.path))
+                detail = _(
+                    "Invalid input for field '%(path)s': %(message)s"
+                ) % {'path': path, 'message': str(ex)}
             else:
-                detail = ex.message
+                detail = str(ex)
             raise exception.SchemaValidationError(detail=detail)
