@@ -12,29 +12,48 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+from oslo_db import api as oslo_db_api
+from sqlalchemy.ext.hybrid import hybrid_property
+
 from keystone.common import driver_hints
 from keystone.common import sql
 from keystone.credential.backends import base
 from keystone import exception
 
 
-class CredentialModel(sql.ModelBase, sql.DictBase):
+class CredentialModel(sql.ModelBase, sql.ModelDictMixinWithExtras):
     __tablename__ = 'credential'
     attributes = [
-        'id', 'user_id', 'project_id', 'encrypted_blob', 'type', 'key_hash'
+        'id',
+        'user_id',
+        'project_id',
+        'encrypted_blob',
+        'type',
+        'key_hash',
     ]
     id = sql.Column(sql.String(64), primary_key=True)
-    user_id = sql.Column(sql.String(64),
-                         nullable=False)
+    user_id = sql.Column(sql.String(64), nullable=False)
     project_id = sql.Column(sql.String(64))
-    encrypted_blob = sql.Column(sql.Text(), nullable=True)
+    _encrypted_blob = sql.Column('encrypted_blob', sql.Text(), nullable=False)
     type = sql.Column(sql.String(255), nullable=False)
-    key_hash = sql.Column(sql.String(64), nullable=True)
+    key_hash = sql.Column(sql.String(64), nullable=False)
     extra = sql.Column(sql.JsonBlob())
 
+    @hybrid_property
+    def encrypted_blob(self):
+        return self._encrypted_blob
 
-class Credential(base.CredentialDriverV8):
+    @encrypted_blob.setter  # type: ignore[no-redef]
+    def encrypted_blob(self, encrypted_blob):
+        # Make sure to hand over the encrypted credential as a string value
+        # to the backend driver to avoid the sql drivers (esp. psycopg2)
+        # treating this as binary data and e.g. hex-escape it.
+        if isinstance(encrypted_blob, bytes):
+            encrypted_blob = encrypted_blob.decode('utf-8')
+        self._encrypted_blob = encrypted_blob
 
+
+class Credential(base.CredentialDriverBase):
     # credential crud
 
     @sql.handle_conflicts(conflict_type='credential')
@@ -48,8 +67,9 @@ class Credential(base.CredentialDriverV8):
     def list_credentials(self, hints):
         with sql.session_for_read() as session:
             credentials = session.query(CredentialModel)
-            credentials = sql.filter_limit_query(CredentialModel,
-                                                 credentials, hints)
+            credentials = sql.filter_limit_query(
+                CredentialModel, credentials, hints
+            )
             return [s.to_dict() for s in credentials]
 
     def list_credentials_for_user(self, user_id, type=None):
@@ -62,7 +82,7 @@ class Credential(base.CredentialDriverV8):
             return [ref.to_dict() for ref in refs]
 
     def _get_credential(self, session, credential_id):
-        ref = session.query(CredentialModel).get(credential_id)
+        ref = session.get(CredentialModel, credential_id)
         if ref is None:
             raise exception.CredentialNotFound(credential_id=credential_id)
         return ref
@@ -96,6 +116,7 @@ class Credential(base.CredentialDriverV8):
             query = query.filter_by(project_id=project_id)
             query.delete()
 
+    @oslo_db_api.wrap_db_retry(retry_on_deadlock=True)
     def delete_credentials_for_user(self, user_id):
         with sql.session_for_write() as session:
             query = session.query(CredentialModel)
