@@ -13,14 +13,12 @@
 from keystone.common import manager
 import keystone.conf
 
-
 CONF = keystone.conf.CONF
 
 
 class Manager(manager.Manager):
-
     driver_namespace = 'keystone.credential.provider'
     _provides_api = 'credential_provider_api'
 
     def __init__(self):
-        super(Manager, self).__init__(CONF.credential.provider)
+        super().__init__(CONF.credential.provider)
