@@ -14,7 +14,6 @@ import hashlib
 
 from cryptography import fernet
 from oslo_log import log
-import six
 
 from keystone.common import fernet_utils
 import keystone.conf
@@ -22,29 +21,28 @@ from keystone.credential.providers import core
 from keystone import exception
 from keystone.i18n import _
 
-
 CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
 # NOTE(lbragstad): Credential key rotation operates slightly different than
-# Fernet key rotation. Each credential holds a hash of the key that encrypted
+# token key rotation. Each credential holds a hash of the key that encrypted
 # it. This is important for credential key rotation because it helps us make
 # sure we don't over-rotate credential keys. During a rotation of credential
 # keys, if any credential has not been re-encrypted with the current primary
 # key, we can abandon the key rotation until all credentials have been migrated
 # to the new primary key. If we don't take this step, it is possible that we
-# could remove a key used to encrypt credentials, leaving them recoverable.
+# could remove a key used to encrypt credentials, leaving them unrecoverable.
 # This also means that we don't need to expose a `[credential] max_active_keys`
-# option through configuration. Instead we will use a global configuration and
-# share that across all places that need to use FernetUtils for credential
+# option through configuration. Instead we will use a global variable and share
+# that across all places that need to use FernetUtils for credential
 # encryption.
 MAX_ACTIVE_KEYS = 3
 
 
 def get_multi_fernet_keys():
     key_utils = fernet_utils.FernetUtils(
-        CONF.credential.key_repository, MAX_ACTIVE_KEYS,
-        'credential')
+        CONF.credential.key_repository, MAX_ACTIVE_KEYS, 'credential'
+    )
     keys = key_utils.load_keys(use_null_key=True)
 
     fernet_keys = [fernet.Fernet(key) for key in keys]
@@ -55,7 +53,7 @@ def get_multi_fernet_keys():
 
 def primary_key_hash(keys):
     """Calculate a hash of the primary key used for encryption."""
-    if isinstance(keys[0], six.text_type):
+    if isinstance(keys[0], str):
         keys[0] = keys[0].encode('utf-8')
     # NOTE(lhinds) This is marked as #nosec since bandit will see SHA1 which
     # is marked as insecure. However, this hash function is used alongside
@@ -78,16 +76,19 @@ class Provider(core.Provider):
                 'Encrypting credentials with the null key. Please properly '
                 'encrypt credentials using `keystone-manage credential_setup`,'
                 ' `keystone-manage credential_migrate`, and `keystone-manage '
-                'credential_rotate`')
+                'credential_rotate`'
+            )
 
         try:
             return (
                 crypto.encrypt(credential.encode('utf-8')),
-                primary_key_hash(keys))
+                primary_key_hash(keys),
+            )
         except (TypeError, ValueError) as e:
-            msg = _('Credential could not be encrypted: %s') % str(e)
+            msg = f'Credential could not be encrypted: {str(e)}'
+            tr_msg = _('Credential could not be encrypted: %s') % str(e)
             LOG.error(msg)
-            raise exception.CredentialEncryptionError(msg)
+            raise exception.CredentialEncryptionError(tr_msg)
 
     def decrypt(self, credential):
         """Attempt to decrypt a credential.
@@ -96,17 +97,24 @@ class Provider(core.Provider):
         :returns: a decrypted credential
         """
         key_utils = fernet_utils.FernetUtils(
-            CONF.credential.key_repository, MAX_ACTIVE_KEYS)
+            CONF.credential.key_repository, MAX_ACTIVE_KEYS, 'credential'
+        )
         keys = key_utils.load_keys(use_null_key=True)
         fernet_keys = [fernet.Fernet(key) for key in keys]
         crypto = fernet.MultiFernet(fernet_keys)
 
         try:
-            if isinstance(credential, six.text_type):
+            if isinstance(credential, str):
                 credential = credential.encode('utf-8')
             return crypto.decrypt(credential).decode('utf-8')
         except (fernet.InvalidToken, TypeError, ValueError):
-            msg = _('Credential could not be decrypted. Please contact the'
-                    ' administrator')
+            msg = (
+                'Credential could not be decrypted. Please contact the '
+                'administrator'
+            )
+            tr_msg = _(
+                'Credential could not be decrypted. Please contact the '
+                'administrator'
+            )
             LOG.error(msg)
-            raise exception.CredentialEncryptionError(msg)
+            raise exception.CredentialEncryptionError(tr_msg)
