@@ -39,17 +39,21 @@ import keystone.conf
 from keystone import exception
 from keystone.i18n import _
 
-
 _CATALOG_HELPER_OBJ = None
 
 LOG = log.getLogger(__name__)
 # NOTE(gyee): actions that can be notified. One must update this list whenever
 # a new action is supported.
 _ACTIONS = collections.namedtuple(
-    'NotificationActions',
-    'created, deleted, disabled, updated, internal')
-ACTIONS = _ACTIONS(created='created', deleted='deleted', disabled='disabled',
-                   updated='updated', internal='internal')
+    '_ACTIONS', ['created', 'deleted', 'disabled', 'updated', 'internal']
+)
+ACTIONS = _ACTIONS(
+    created='created',
+    deleted='deleted',
+    disabled='disabled',
+    updated='updated',
+    internal='internal',
+)
 """The actions on resources."""
 
 CADF_TYPE_MAP = {
@@ -71,7 +75,7 @@ CADF_TYPE_MAP = {
 
 SAML_AUDIT_TYPE = 'http://docs.oasis-open.org/security/saml/v2.0'
 # resource types that can be notified
-_SUBSCRIBERS = {}
+_SUBSCRIBERS: dict = {}
 _notifier = None
 SERVICE = 'identity'
 PROVIDERS = provider_api.ProviderAPIs
@@ -90,10 +94,12 @@ DOMAIN_DELETED = 'domain_deleted'
 
 def build_audit_initiator():
     """A pyCADF initiator describing the current authenticated context."""
-    pycadf_host = host.Host(address=flask.request.remote_addr,
-                            agent=str(flask.request.user_agent))
-    initiator = resource.Resource(typeURI=taxonomy.ACCOUNT_USER,
-                                  host=pycadf_host)
+    pycadf_host = host.Host(
+        address=flask.request.remote_addr, agent=str(flask.request.user_agent)
+    )
+    initiator = resource.Resource(
+        typeURI=taxonomy.ACCOUNT_USER, host=pycadf_host
+    )
     oslo_context = flask.request.environ.get(context.REQUEST_CONTEXT_ENV)
     if oslo_context.user_id:
         initiator.id = utils.resource_uuid(oslo_context.user_id)
@@ -113,7 +119,7 @@ def build_audit_initiator():
     return initiator
 
 
-class Audit(object):
+class Audit:
     """Namespace for audit notification functions.
 
     This is a namespace object to contain all of the direct notification
@@ -121,8 +127,16 @@ class Audit(object):
     """
 
     @classmethod
-    def _emit(cls, operation, resource_type, resource_id, initiator, public,
-              actor_dict=None, reason=None):
+    def _emit(
+        cls,
+        operation,
+        resource_type,
+        resource_id,
+        initiator,
+        public,
+        actor_dict=None,
+        reason=None,
+    ):
         """Directly send an event notification.
 
         :param operation: one of the values from ACTIONS
@@ -149,54 +163,143 @@ class Audit(object):
             resource_id,
             initiator=initiator,
             actor_dict=actor_dict,
-            public=public)
+            public=public,
+        )
 
         if CONF.notification_format == 'cadf' and public:
             outcome = taxonomy.OUTCOME_SUCCESS
-            _create_cadf_payload(operation, resource_type, resource_id,
-                                 outcome, initiator, reason)
+            _create_cadf_payload(
+                operation,
+                resource_type,
+                resource_id,
+                outcome,
+                initiator,
+                reason,
+            )
 
     @classmethod
-    def created(cls, resource_type, resource_id, initiator=None,
-                public=True, reason=None):
-        cls._emit(ACTIONS.created, resource_type, resource_id, initiator,
-                  public, reason=reason)
+    def created(
+        cls,
+        resource_type,
+        resource_id,
+        initiator=None,
+        public=True,
+        reason=None,
+    ):
+        cls._emit(
+            ACTIONS.created,
+            resource_type,
+            resource_id,
+            initiator,
+            public,
+            reason=reason,
+        )
 
     @classmethod
-    def updated(cls, resource_type, resource_id, initiator=None,
-                public=True, reason=None):
-        cls._emit(ACTIONS.updated, resource_type, resource_id, initiator,
-                  public, reason=reason)
+    def updated(
+        cls,
+        resource_type,
+        resource_id,
+        initiator=None,
+        public=True,
+        reason=None,
+    ):
+        cls._emit(
+            ACTIONS.updated,
+            resource_type,
+            resource_id,
+            initiator,
+            public,
+            reason=reason,
+        )
 
     @classmethod
-    def disabled(cls, resource_type, resource_id, initiator=None,
-                 public=True, reason=None):
-        cls._emit(ACTIONS.disabled, resource_type, resource_id, initiator,
-                  public, reason=reason)
+    def disabled(
+        cls,
+        resource_type,
+        resource_id,
+        initiator=None,
+        public=True,
+        reason=None,
+    ):
+        cls._emit(
+            ACTIONS.disabled,
+            resource_type,
+            resource_id,
+            initiator,
+            public,
+            reason=reason,
+        )
 
     @classmethod
-    def deleted(cls, resource_type, resource_id, initiator=None,
-                public=True, reason=None):
-        cls._emit(ACTIONS.deleted, resource_type, resource_id, initiator,
-                  public, reason=reason)
+    def deleted(
+        cls,
+        resource_type,
+        resource_id,
+        initiator=None,
+        public=True,
+        reason=None,
+    ):
+        cls._emit(
+            ACTIONS.deleted,
+            resource_type,
+            resource_id,
+            initiator,
+            public,
+            reason=reason,
+        )
 
     @classmethod
-    def added_to(cls, target_type, target_id, actor_type, actor_id,
-                 initiator=None, public=True, reason=None):
-        actor_dict = {'id': actor_id,
-                      'type': actor_type,
-                      'actor_operation': 'added'}
-        cls._emit(ACTIONS.updated, target_type, target_id, initiator, public,
-                  actor_dict=actor_dict, reason=reason)
+    def added_to(
+        cls,
+        target_type,
+        target_id,
+        actor_type,
+        actor_id,
+        initiator=None,
+        public=True,
+        reason=None,
+    ):
+        actor_dict = {
+            'id': actor_id,
+            'type': actor_type,
+            'actor_operation': 'added',
+        }
+        cls._emit(
+            ACTIONS.updated,
+            target_type,
+            target_id,
+            initiator,
+            public,
+            actor_dict=actor_dict,
+            reason=reason,
+        )
 
     @classmethod
-    def removed_from(cls, target_type, target_id, actor_type, actor_id,
-                     initiator=None, public=True, reason=None):
-        actor_dict = {'id': actor_id,
-                      'type': actor_type,
-                      'actor_operation': 'removed'}
-        cls._emit(ACTIONS.updated, target_type, target_id, initiator, public,
-                  actor_dict=actor_dict, reason=reason)
+    def removed_from(
+        cls,
+        target_type,
+        target_id,
+        actor_type,
+        actor_id,
+        initiator=None,
+        public=True,
+        reason=None,
+    ):
+        actor_dict = {
+            'id': actor_id,
+            'type': actor_type,
+            'actor_operation': 'removed',
+        }
+        cls._emit(
+            ACTIONS.updated,
+            target_type,
+            target_id,
+            initiator,
+            public,
+            actor_dict=actor_dict,
+            reason=reason,
+        )
 
     @classmethod
     def internal(cls, resource_type, resource_id, reason=None):
@@ -207,8 +310,14 @@ class Audit(object):
         # internal notification publicly.
         initiator = None
         public = False
-        cls._emit(ACTIONS.internal, resource_type, resource_id, initiator,
-                  public, reason)
+        cls._emit(
+            ACTIONS.internal,
+            resource_type,
+            resource_id,
+            initiator,
+            public,
+            reason,
+        )
 
 
 def invalidate_token_cache_notification(reason):
@@ -232,8 +341,12 @@ def invalidate_token_cache_notification(reason):
     initiator = None
     public = False
     Audit._emit(
-        ACTIONS.internal, INVALIDATE_TOKEN_CACHE, resource_id, initiator,
-        public, reason=reason
+        ACTIONS.internal,
+        INVALIDATE_TOKEN_CACHE,
+        resource_id,
+        initiator,
+        public,
+        reason=reason,
     )
 
 
@@ -250,8 +363,9 @@ def _get_callback_info(callback):
     module_name = getattr(callback, '__module__', None)
     func_name = callback.__name__
     if inspect.ismethod(callback):
-        class_name = reflection.get_class_name(callback.__self__,
-                                               fully_qualified=False)
+        class_name = reflection.get_class_name(
+            callback.__self__, fully_qualified=False
+        )
         return [module_name, class_name, func_name]
     else:
         return [module_name, func_name]
@@ -270,16 +384,20 @@ def register_event_callback(event, resource_type, callbacks):
     :raises TypeError: If callback is not callable
     """
     if event not in ACTIONS:
-        raise ValueError(_('%(event)s is not a valid notification event, must '
-                           'be one of: %(actions)s') %
-                         {'event': event, 'actions': ', '.join(ACTIONS)})
+        raise ValueError(
+            _(
+                '%(event)s is not a valid notification event, must '
+                'be one of: %(actions)s'
+            )
+            % {'event': event, 'actions': ', '.join(ACTIONS)}
+        )
 
     if not hasattr(callbacks, '__iter__'):
         callbacks = [callbacks]
 
     for callback in callbacks:
         if not callable(callback):
-            msg = 'Method not callable: %s' % callback
+            msg = f'Method not callable: {callback}'
             tr_msg = _('Method not callable: %s') % callback
             LOG.error(msg)
             raise TypeError(tr_msg)
@@ -311,11 +429,10 @@ def listener(cls):
 
         @listener
         class Something(object):
-
             def __init__(self):
                 self.event_callbacks = {
                     notifications.ACTIONS.created: {
-                        'user': self._user_created_callback,
+                        'user': self._user_created_callback
                     },
                     notifications.ACTIONS.deleted: {
                         'project': [
@@ -326,11 +443,13 @@ def listener(cls):
                 }
 
     """
+
     def init_wrapper(init):
         @functools.wraps(init)
         def __new_init__(self, *args, **kwargs):
             init(self, *args, **kwargs)
             _register_event_callbacks(self)
+
         return __new_init__
 
     def _register_event_callbacks(self):
@@ -347,14 +466,19 @@ def notify_event_callbacks(service, resource_type, operation, payload):
     if operation in _SUBSCRIBERS:
         if resource_type in _SUBSCRIBERS[operation]:
             for cb in _SUBSCRIBERS[operation][resource_type]:
-                subst_dict = {'cb_name': cb.__name__,
-                              'service': service,
-                              'resource_type': resource_type,
-                              'operation': operation,
-                              'payload': payload}
-                LOG.debug('Invoking callback %(cb_name)s for event '
-                          '%(service)s %(resource_type)s %(operation)s for '
-                          '%(payload)s', subst_dict)
+                subst_dict = {
+                    'cb_name': cb.__name__,
+                    'service': service,
+                    'resource_type': resource_type,
+                    'operation': operation,
+                    'payload': payload,
+                }
+                LOG.debug(
+                    'Invoking callback %(cb_name)s for event '
+                    '%(service)s %(resource_type)s %(operation)s for '
+                    '%(payload)s',
+                    subst_dict,
+                )
                 cb(service, resource_type, operation, payload)
 
 
@@ -372,8 +496,7 @@ def _get_notifier():
         host = CONF.default_publisher_id or socket.gethostname()
         try:
             transport = oslo_messaging.get_notification_transport(CONF)
-            _notifier = oslo_messaging.Notifier(transport,
-                                                "identity.%s" % host)
+            _notifier = oslo_messaging.Notifier(transport, f"identity.{host}")
         except Exception:
             LOG.exception("Failed to construct notifier")
             _notifier = False
@@ -400,8 +523,9 @@ def reset_notifier():
     _notifier = None
 
 
-def _create_cadf_payload(operation, resource_type, resource_id,
-                         outcome, initiator, reason=None):
+def _create_cadf_payload(
+    operation, resource_type, resource_id, outcome, initiator, reason=None
+):
     """Prepare data for CADF audit notifier.
 
     Transform the arguments into content to be consumed by the function that
@@ -436,19 +560,31 @@ def _create_cadf_payload(operation, resource_type, resource_id,
     if resource_id == ROOT_DOMAIN:
         return
 
-    target = resource.Resource(typeURI=target_uri,
-                               id=resource_id)
+    target = resource.Resource(typeURI=target_uri, id=resource_id)
 
     audit_kwargs = {'resource_info': resource_id}
-    cadf_action = '%s.%s' % (operation, resource_type)
-    event_type = '%s.%s.%s' % (SERVICE, resource_type, operation)
+    cadf_action = f'{operation}.{resource_type}'
+    event_type = f'{SERVICE}.{resource_type}.{operation}'
 
-    _send_audit_notification(cadf_action, initiator, outcome,
-                             target, event_type, reason=reason, **audit_kwargs)
+    _send_audit_notification(
+        cadf_action,
+        initiator,
+        outcome,
+        target,
+        event_type,
+        reason=reason,
+        **audit_kwargs,
+    )
 
 
-def _send_notification(operation, resource_type, resource_id, initiator=None,
-                       actor_dict=None, public=True):
+def _send_notification(
+    operation,
+    resource_type,
+    resource_id,
+    initiator=None,
+    actor_dict=None,
+    public=True,
+):
     """Send notification to inform observers about the affected resource.
 
     This method doesn't raise an exception when sending the notification fails.
@@ -485,10 +621,7 @@ def _send_notification(operation, resource_type, resource_id, initiator=None,
         notifier = _get_notifier()
         if notifier:
             context = {}
-            event_type = '%(service)s.%(resource_type)s.%(operation)s' % {
-                'service': SERVICE,
-                'resource_type': resource_type,
-                'operation': operation}
+            event_type = f'{SERVICE}.{resource_type}.{operation}'
             if _check_notification_opt_out(event_type, outcome=None):
                 return
             try:
@@ -496,7 +629,8 @@ def _send_notification(operation, resource_type, resource_id, initiator=None,
             except Exception:
                 LOG.exception(
                     'Failed to send %(res_id)s %(event_type)s notification',
-                    {'res_id': resource_id, 'event_type': event_type})
+                    {'res_id': resource_id, 'event_type': event_type},
+                )
 
 
 def _get_request_audit_info(context, user_id=None):
@@ -517,12 +651,15 @@ def _get_request_audit_info(context, user_id=None):
         remote_addr = environment.get('REMOTE_ADDR')
         http_user_agent = environment.get('HTTP_USER_AGENT')
         if not user_id:
-            user_id = environment.get('KEYSTONE_AUTH_CONTEXT',
-                                      {}).get('user_id')
-        project_id = environment.get('KEYSTONE_AUTH_CONTEXT',
-                                     {}).get('project_id')
-        domain_id = environment.get('KEYSTONE_AUTH_CONTEXT',
-                                    {}).get('domain_id')
+            user_id = environment.get('KEYSTONE_AUTH_CONTEXT', {}).get(
+                'user_id'
+            )
+        project_id = environment.get('KEYSTONE_AUTH_CONTEXT', {}).get(
+            'project_id'
+        )
+        domain_id = environment.get('KEYSTONE_AUTH_CONTEXT', {}).get(
+            'domain_id'
+        )
 
     host = pycadf.host.Host(address=remote_addr, agent=http_user_agent)
     initiator = resource.Resource(typeURI=taxonomy.ACCOUNT_USER, host=host)
@@ -540,7 +677,7 @@ def _get_request_audit_info(context, user_id=None):
     return initiator
 
 
-class CadfNotificationWrapper(object):
+class CadfNotificationWrapper:
     """Send CADF event notifications for various methods.
 
     This function is only used for Authentication events. Its ``action`` and
@@ -558,7 +695,7 @@ class CadfNotificationWrapper(object):
 
     def __init__(self, operation):
         self.action = operation
-        self.event_type = '%s.%s' % (SERVICE, operation)
+        self.event_type = f'{SERVICE}.{operation}'
 
     def __call__(self, f):
         @functools.wraps(f)
@@ -571,32 +708,45 @@ class CadfNotificationWrapper(object):
             initiator.id = utils.resource_uuid(user_id)
             try:
                 result = f(wrapped_self, user_id, *args, **kwargs)
-            except (exception.AccountLocked,
-                    exception.PasswordExpired) as ex:
+            except (exception.AccountLocked, exception.PasswordExpired) as ex:
                 # Send a CADF event with a reason for PCI-DSS related
                 # authentication failures
                 audit_reason = reason.Reason(str(ex), str(ex.code))
-                _send_audit_notification(self.action, initiator,
-                                         taxonomy.OUTCOME_FAILURE,
-                                         target, self.event_type,
-                                         reason=audit_reason)
+                _send_audit_notification(
+                    self.action,
+                    initiator,
+                    taxonomy.OUTCOME_FAILURE,
+                    target,
+                    self.event_type,
+                    reason=audit_reason,
+                )
+                if isinstance(ex, exception.AccountLocked):
+                    raise exception.Unauthorized
                 raise
             except Exception:
                 # For authentication failure send a CADF event as well
-                _send_audit_notification(self.action, initiator,
-                                         taxonomy.OUTCOME_FAILURE,
-                                         target, self.event_type)
+                _send_audit_notification(
+                    self.action,
+                    initiator,
+                    taxonomy.OUTCOME_FAILURE,
+                    target,
+                    self.event_type,
+                )
                 raise
             else:
-                _send_audit_notification(self.action, initiator,
-                                         taxonomy.OUTCOME_SUCCESS,
-                                         target, self.event_type)
+                _send_audit_notification(
+                    self.action,
+                    initiator,
+                    taxonomy.OUTCOME_SUCCESS,
+                    target,
+                    self.event_type,
+                )
                 return result
 
         return wrapper
 
 
-class CadfRoleAssignmentNotificationWrapper(object):
+class CadfRoleAssignmentNotificationWrapper:
     """Send CADF notifications for ``role_assignment`` methods.
 
     This function is only used for role assignment events. Its ``action`` and
@@ -615,9 +765,8 @@ class CadfRoleAssignmentNotificationWrapper(object):
     ROLE_ASSIGNMENT = 'role_assignment'
 
     def __init__(self, operation):
-        self.action = '%s.%s' % (operation, self.ROLE_ASSIGNMENT)
-        self.event_type = '%s.%s.%s' % (SERVICE, self.ROLE_ASSIGNMENT,
-                                        operation)
+        self.action = f'{operation}.{self.ROLE_ASSIGNMENT}'
+        self.event_type = f'{SERVICE}.{self.ROLE_ASSIGNMENT}.{operation}'
 
     def __call__(self, f):
         @functools.wraps(f)
@@ -630,8 +779,9 @@ class CadfRoleAssignmentNotificationWrapper(object):
             differently in various tests.
             Using named arguments, i.e.::
 
-                create_grant(user_id=user['id'], domain_id=domain['id'],
-                             role_id=role['id'])
+                create_grant(
+                    user_id=user['id'], domain_id=domain['id'], role_id=role['id']
+                )
 
             Or, using positional arguments, i.e.::
 
@@ -640,8 +790,9 @@ class CadfRoleAssignmentNotificationWrapper(object):
 
             Or, both, i.e.::
 
-                create_grant(role_id['id'], user_id=user['id'],
-                             domain_id=domain['id'])
+                create_grant(
+                    role_id['id'], user_id=user['id'], domain_id=domain['id']
+                )
 
             Checking the values for kwargs is easy enough, since it comes
             in as a dictionary
@@ -650,16 +801,22 @@ class CadfRoleAssignmentNotificationWrapper(object):
 
             ::
 
-                create_grant(role_id, user_id=None, group_id=None,
-                             domain_id=None, project_id=None,
-                             inherited_to_projects=False)
+                create_grant(
+                    role_id,
+                    user_id=None,
+                    group_id=None,
+                    domain_id=None,
+                    project_id=None,
+                    inherited_to_projects=False,
+                )
 
             So, if the values of actor or target are still None after
             checking kwargs, we can check the positional arguments,
             based on the method signature.
             """
             call_args = inspect.getcallargs(
-                f, wrapped_self, role_id, *args, **kwargs)
+                f, wrapped_self, role_id, *args, **kwargs
+            )
             inherited = call_args['inherited_to_projects']
             initiator = call_args.get('initiator', None)
             target = resource.Resource(typeURI=taxonomy.ACCOUNT_USER)
@@ -681,24 +838,32 @@ class CadfRoleAssignmentNotificationWrapper(object):
             try:
                 result = f(wrapped_self, role_id, *args, **kwargs)
             except Exception:
-                _send_audit_notification(self.action, initiator,
-                                         taxonomy.OUTCOME_FAILURE,
-                                         target, self.event_type,
-                                         **audit_kwargs)
+                _send_audit_notification(
+                    self.action,
+                    initiator,
+                    taxonomy.OUTCOME_FAILURE,
+                    target,
+                    self.event_type,
+                    **audit_kwargs,
+                )
                 raise
             else:
-                _send_audit_notification(self.action, initiator,
-                                         taxonomy.OUTCOME_SUCCESS,
-                                         target, self.event_type,
-                                         **audit_kwargs)
+                _send_audit_notification(
+                    self.action,
+                    initiator,
+                    taxonomy.OUTCOME_SUCCESS,
+                    target,
+                    self.event_type,
+                    **audit_kwargs,
+                )
                 return result
 
         return wrapper
 
 
-def send_saml_audit_notification(action, user_id, group_ids,
-                                 identity_provider, protocol, token_id,
-                                 outcome):
+def send_saml_audit_notification(
+    action, user_id, group_ids, identity_provider, protocol, token_id, outcome
+):
     """Send notification to inform observers about SAML events.
 
     :param action: Action being audited
@@ -722,20 +887,25 @@ def send_saml_audit_notification(action, user_id, group_ids,
     user_id = user_id or taxonomy.UNKNOWN
     token_id = token_id or taxonomy.UNKNOWN
     group_ids = group_ids or []
-    cred = credential.FederatedCredential(token=token_id, type=audit_type,
-                                          identity_provider=identity_provider,
-                                          user=user_id, groups=group_ids)
+    cred = credential.FederatedCredential(
+        token=token_id,
+        type=audit_type,
+        identity_provider=identity_provider,
+        user=user_id,
+        groups=group_ids,
+    )
     initiator.credential = cred
-    event_type = '%s.%s' % (SERVICE, action)
+    event_type = f'{SERVICE}.{action}'
     _send_audit_notification(action, initiator, outcome, target, event_type)
 
 
-class _CatalogHelperObj(provider_api.ProviderAPIMixin, object):
+class _CatalogHelperObj(provider_api.ProviderAPIMixin):
     """A helper object to allow lookups of identity service id."""
 
 
-def _send_audit_notification(action, initiator, outcome, target,
-                             event_type, reason=None, **kwargs):
+def _send_audit_notification(
+    action, initiator, outcome, target, event_type, reason=None, **kwargs
+):
     """Send CADF notification to inform observers about the affected resource.
 
     This method logs an exception when sending the notification fails.
@@ -775,7 +945,8 @@ def _send_audit_notification(action, initiator, outcome, target,
         initiator=initiator,
         target=target,
         reason=reason,
-        observer=resource.Resource(typeURI=taxonomy.SERVICE_SECURITY))
+        observer=resource.Resource(typeURI=taxonomy.SERVICE_SECURITY),
+    )
 
     if service_id is not None:
         event.observer.id = service_id
@@ -795,7 +966,8 @@ def _send_audit_notification(action, initiator, outcome, target,
             # notification should not interfere with the API request
             LOG.exception(
                 'Failed to send %(action)s %(event_type)s notification',
-                {'action': action, 'event_type': event_type})
+                {'action': action, 'event_type': event_type},
+            )
 
 
 def _check_notification_opt_out(event_type, outcome):
