@@ -21,16 +21,13 @@ from keystone.common.rbac_enforcer import policy
 from keystone import exception
 from keystone.policy.backends import base
 
-
 LOG = log.getLogger(__name__)
 
 
 class Policy(base.PolicyDriverBase):
     def enforce(self, credentials, action, target):
         msg = 'enforce %(action)s: %(credentials)s'
-        LOG.debug(msg, {
-            'action': action,
-            'credentials': credentials})
+        LOG.debug(msg, {'action': action, 'credentials': credentials})
         policy.enforce(credentials, action, target)
 
     def create_policy(self, policy_id, policy):
