@@ -27,7 +27,6 @@ class PolicyModel(sql.ModelBase, sql.ModelDictMixinWithExtras):
 
 
 class Policy(rules.Policy):
-
     @sql.handle_conflicts(conflict_type='policy')
     def create_policy(self, policy_id, policy):
         with sql.session_for_write() as session:
@@ -43,7 +42,7 @@ class Policy(rules.Policy):
 
     def _get_policy(self, session, policy_id):
         """Private method to get a policy model object (NOT a dictionary)."""
-        ref = session.query(PolicyModel).get(policy_id)
+        ref = session.get(PolicyModel, policy_id)
         if not ref:
             raise exception.PolicyNotFound(policy_id=policy_id)
         return ref
