@@ -10,9 +10,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import uuid
-
 import http.client
+import uuid
 
 from keystone.common import provider_api
 import keystone.conf
@@ -25,12 +24,13 @@ CONF = keystone.conf.CONF
 PROVIDERS = provider_api.ProviderAPIs
 
 
-class _SystemUserEndpointGroupsTests(object):
+class _SystemUserEndpointGroupsTests:
     """Common default functionality for all system users."""
 
     def test_user_can_list_endpoint_groups(self):
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
@@ -45,30 +45,40 @@ class _SystemUserEndpointGroupsTests(object):
 
     def test_user_can_get_an_endpoint_group(self):
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         with self.test_client() as c:
-            c.get('/v3/OS-EP-FILTER/endpoint_groups/%s' % endpoint_group['id'],
-                  headers=self.headers)
+            c.get(
+                '/v3/OS-EP-FILTER/endpoint_groups/{}'.format(
+                    endpoint_group['id']
+                ),
+                headers=self.headers,
+            )
 
     def test_user_can_list_projects_associated_with_endpoint_groups(self):
         project = PROVIDERS.resource_api.create_project(
-            uuid.uuid4().hex, unit.new_project_ref(
-                domain_id=CONF.identity.default_domain_id
-            )
+            uuid.uuid4().hex,
+            unit.new_project_ref(domain_id=CONF.identity.default_domain_id),
         )
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         PROVIDERS.catalog_api.add_endpoint_group_to_project(
-            endpoint_group['id'], project['id'])
+            endpoint_group['id'], project['id']
+        )
         with self.test_client() as c:
-            r = c.get('/v3/OS-EP-FILTER/endpoint_groups/%s/projects'
-                      % endpoint_group['id'], headers=self.headers)
+            r = c.get(
+                '/v3/OS-EP-FILTER/endpoint_groups/{}/projects'.format(
+                    endpoint_group['id']
+                ),
+                headers=self.headers,
+            )
             projects = []
             for project in r.json['projects']:
                 projects.append(project['id'])
@@ -83,14 +93,18 @@ class _SystemUserEndpointGroupsTests(object):
             endpoint['id'], endpoint
         )
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         with self.test_client() as c:
-            r = c.get('/v3/OS-EP-FILTER/endpoint_groups/%s/endpoints'
-                      % endpoint_group['id'],
-                      headers=self.headers)
+            r = c.get(
+                '/v3/OS-EP-FILTER/endpoint_groups/{}/endpoints'.format(
+                    endpoint_group['id']
+                ),
+                headers=self.headers,
+            )
             endpoints = []
             for endpoint in r.json['endpoints']:
                 endpoints.append(endpoint['id'])
@@ -98,45 +112,53 @@ class _SystemUserEndpointGroupsTests(object):
 
     def test_user_can_get_endpoints_associated_with_endpoint_groups(self):
         project = PROVIDERS.resource_api.create_project(
-            uuid.uuid4().hex, unit.new_project_ref(
-                domain_id=CONF.identity.default_domain_id
-            )
+            uuid.uuid4().hex,
+            unit.new_project_ref(domain_id=CONF.identity.default_domain_id),
         )
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         PROVIDERS.catalog_api.add_endpoint_group_to_project(
-            endpoint_group['id'], project['id'])
+            endpoint_group['id'], project['id']
+        )
         with self.test_client() as c:
-            c.get('/v3/OS-EP-FILTER/endpoint_groups/%s/projects/%s'
-                  % (endpoint_group['id'], project['id']),
-                  headers=self.headers)
+            c.get(
+                '/v3/OS-EP-FILTER/endpoint_groups/{}/projects/{}'.format(
+                    endpoint_group['id'], project['id']
+                ),
+                headers=self.headers,
+            )
 
     def test_user_can_list_endpoint_groups_with_their_projects(self):
         project = PROVIDERS.resource_api.create_project(
-            uuid.uuid4().hex, unit.new_project_ref(
-                domain_id=CONF.identity.default_domain_id
-            )
+            uuid.uuid4().hex,
+            unit.new_project_ref(domain_id=CONF.identity.default_domain_id),
         )
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         PROVIDERS.catalog_api.add_endpoint_group_to_project(
-            endpoint_group['id'], project['id'])
+            endpoint_group['id'], project['id']
+        )
         with self.test_client() as c:
-            r = c.get('/v3/OS-EP-FILTER/projects/%s/endpoint_groups'
-                      % project['id'],
-                      headers=self.headers)
+            r = c.get(
+                '/v3/OS-EP-FILTER/projects/{}/endpoint_groups'.format(
+                    project['id']
+                ),
+                headers=self.headers,
+            )
             endpoint_groups = []
             for endpoint_group in r.json['endpoint_groups']:
                 endpoint_groups.append(endpoint_group['id'])
 
 
-class _SystemReaderAndMemberUserEndpointGroupsTests(object):
+class _SystemReaderAndMemberUserEndpointGroupsTests:
     """Common default functionality for system readers and system members."""
 
     def test_user_cannot_create_endpoint_groups(self):
@@ -145,20 +167,22 @@ class _SystemReaderAndMemberUserEndpointGroupsTests(object):
                 'id': uuid.uuid4().hex,
                 'description': uuid.uuid4().hex,
                 'filters': {'interface': 'public'},
-                'name': uuid.uuid4().hex
+                'name': uuid.uuid4().hex,
             }
         }
 
         with self.test_client() as c:
             c.post(
-                '/v3/OS-EP-FILTER/endpoint_groups', json=create,
+                '/v3/OS-EP-FILTER/endpoint_groups',
+                json=create,
                 headers=self.headers,
-                expected_status_code=http.client.FORBIDDEN
+                expected_status_code=http.client.FORBIDDEN,
             )
 
     def test_user_cannot_update_endpoint_groups(self):
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
@@ -167,105 +191,126 @@ class _SystemReaderAndMemberUserEndpointGroupsTests(object):
 
         with self.test_client() as c:
             c.patch(
-                '/v3/OS-EP-FILTER/endpoint_groups/%s' % endpoint_group['id'],
+                '/v3/OS-EP-FILTER/endpoint_groups/{}'.format(
+                    endpoint_group['id']
+                ),
                 json=update,
                 headers=self.headers,
-                expected_status_code=http.client.FORBIDDEN
+                expected_status_code=http.client.FORBIDDEN,
             )
 
     def test_user_cannot_delete_endpoint_groups(self):
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
 
         with self.test_client() as c:
             c.delete(
-                '/v3/OS-EP-FILTER/endpoint_groups/%s' % endpoint_group['id'],
+                '/v3/OS-EP-FILTER/endpoint_groups/{}'.format(
+                    endpoint_group['id']
+                ),
                 headers=self.headers,
-                expected_status_code=http.client.FORBIDDEN
+                expected_status_code=http.client.FORBIDDEN,
             )
 
     def test_user_cannot_add_endpoint_group_to_project(self):
         project = PROVIDERS.resource_api.create_project(
-            uuid.uuid4().hex, unit.new_project_ref(
-                domain_id=CONF.identity.default_domain_id
-            )
+            uuid.uuid4().hex,
+            unit.new_project_ref(domain_id=CONF.identity.default_domain_id),
         )
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         with self.test_client() as c:
-            c.put('/v3/OS-EP-FILTER/endpoint_groups/%s/projects/%s'
-                  % (endpoint_group['id'], project['id']),
-                  headers=self.headers,
-                  expected_status_code=http.client.FORBIDDEN
-                  )
+            c.put(
+                '/v3/OS-EP-FILTER/endpoint_groups/{}/projects/{}'.format(
+                    endpoint_group['id'], project['id']
+                ),
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
+            )
 
     def test_cannot_remove_endpoint_group_from_project(self):
         project = PROVIDERS.resource_api.create_project(
-            uuid.uuid4().hex, unit.new_project_ref(
-                domain_id=CONF.identity.default_domain_id
-            )
+            uuid.uuid4().hex,
+            unit.new_project_ref(domain_id=CONF.identity.default_domain_id),
         )
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         with self.test_client() as c:
-            c.delete('/v3/OS-EP-FILTER/endpoint_groups/%s/projects/%s'
-                     % (endpoint_group['id'], project['id']),
-                     headers=self.headers,
-                     expected_status_code=http.client.FORBIDDEN
-                     )
+            c.delete(
+                '/v3/OS-EP-FILTER/endpoint_groups/{}/projects/{}'.format(
+                    endpoint_group['id'], project['id']
+                ),
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
+            )
 
 
-class _DomainAndProjectUserEndpointGroupTests(object):
-
+class _DomainAndProjectUserEndpointGroupTests:
     def test_user_cannot_list_endpoint_groups(self):
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
 
         with self.test_client() as c:
-            c.get('/v3/OS-EP-FILTER/endpoint_groups', headers=self.headers,
-                  expected_status_code=http.client.FORBIDDEN)
+            c.get(
+                '/v3/OS-EP-FILTER/endpoint_groups',
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
+            )
 
     def test_user_cannot_get_an_endpoint_group(self):
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         with self.test_client() as c:
-            c.get('/v3/OS-EP-FILTER/endpoint_groups/%s' % endpoint_group['id'],
-                  headers=self.headers,
-                  expected_status_code=http.client.FORBIDDEN)
+            c.get(
+                '/v3/OS-EP-FILTER/endpoint_groups/{}'.format(
+                    endpoint_group['id']
+                ),
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
+            )
 
     def test_user_cannot_list_projects_associated_with_endpoint_groups(self):
         project = PROVIDERS.resource_api.create_project(
-            uuid.uuid4().hex, unit.new_project_ref(
-                domain_id=CONF.identity.default_domain_id
-            )
+            uuid.uuid4().hex,
+            unit.new_project_ref(domain_id=CONF.identity.default_domain_id),
         )
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         PROVIDERS.catalog_api.add_endpoint_group_to_project(
-            endpoint_group['id'], project['id'])
+            endpoint_group['id'], project['id']
+        )
         with self.test_client() as c:
-            c.get('/v3/OS-EP-FILTER/endpoint_groups/%s/projects'
-                  % endpoint_group['id'],
-                  headers=self.headers,
-                  expected_status_code=http.client.FORBIDDEN)
+            c.get(
+                '/v3/OS-EP-FILTER/endpoint_groups/{}/projects'.format(
+                    endpoint_group['id']
+                ),
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
+            )
 
     def test_user_cannot_list_endpoints_associated_with_endpoint_groups(self):
         service = PROVIDERS.catalog_api.create_service(
@@ -276,53 +321,65 @@ class _DomainAndProjectUserEndpointGroupTests(object):
             endpoint['id'], endpoint
         )
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         with self.test_client() as c:
-            c.get('/v3/OS-EP-FILTER/endpoint_groups/%s/endpoints'
-                  % endpoint_group['id'],
-                  headers=self.headers,
-                  expected_status_code=http.client.FORBIDDEN)
+            c.get(
+                '/v3/OS-EP-FILTER/endpoint_groups/{}/endpoints'.format(
+                    endpoint_group['id']
+                ),
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
+            )
 
     def test_user_cannot_get_endpoints_associated_with_endpoint_groups(self):
         project = PROVIDERS.resource_api.create_project(
-            uuid.uuid4().hex, unit.new_project_ref(
-                domain_id=CONF.identity.default_domain_id
-            )
+            uuid.uuid4().hex,
+            unit.new_project_ref(domain_id=CONF.identity.default_domain_id),
         )
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         PROVIDERS.catalog_api.add_endpoint_group_to_project(
-            endpoint_group['id'], project['id'])
+            endpoint_group['id'], project['id']
+        )
         with self.test_client() as c:
-            c.get('/v3/OS-EP-FILTER/endpoint_groups/%s/projects/%s'
-                  % (endpoint_group['id'], project['id']),
-                  headers=self.headers,
-                  expected_status_code=http.client.FORBIDDEN)
+            c.get(
+                '/v3/OS-EP-FILTER/endpoint_groups/{}/projects/{}'.format(
+                    endpoint_group['id'], project['id']
+                ),
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
+            )
 
     def test_user_cannot_list_endpoint_groups_with_their_projects(self):
         project = PROVIDERS.resource_api.create_project(
-            uuid.uuid4().hex, unit.new_project_ref(
-                domain_id=CONF.identity.default_domain_id
-            )
+            uuid.uuid4().hex,
+            unit.new_project_ref(domain_id=CONF.identity.default_domain_id),
         )
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         PROVIDERS.catalog_api.add_endpoint_group_to_project(
-            endpoint_group['id'], project['id'])
+            endpoint_group['id'], project['id']
+        )
         with self.test_client() as c:
-            c.get('/v3/OS-EP-FILTER/projects/%s/endpoint_groups'
-                  % project['id'],
-                  headers=self.headers,
-                  expected_status_code=http.client.FORBIDDEN)
+            c.get(
+                '/v3/OS-EP-FILTER/projects/{}/endpoint_groups'.format(
+                    project['id']
+                ),
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
+            )
 
     def test_user_cannot_create_endpoint_groups(self):
         create = {
@@ -330,20 +387,22 @@ class _DomainAndProjectUserEndpointGroupTests(object):
                 'id': uuid.uuid4().hex,
                 'description': uuid.uuid4().hex,
                 'filters': {'interface': 'public'},
-                'name': uuid.uuid4().hex
+                'name': uuid.uuid4().hex,
             }
         }
 
         with self.test_client() as c:
             c.post(
-                '/v3/OS-EP-FILTER/endpoint_groups', json=create,
+                '/v3/OS-EP-FILTER/endpoint_groups',
+                json=create,
                 headers=self.headers,
-                expected_status_code=http.client.FORBIDDEN
+                expected_status_code=http.client.FORBIDDEN,
             )
 
     def test_user_cannot_update_endpoint_groups(self):
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
@@ -352,70 +411,80 @@ class _DomainAndProjectUserEndpointGroupTests(object):
 
         with self.test_client() as c:
             c.patch(
-                '/v3/OS-EP-FILTER/endpoint_groups/%s' % endpoint_group['id'],
+                '/v3/OS-EP-FILTER/endpoint_groups/{}'.format(
+                    endpoint_group['id']
+                ),
                 json=update,
                 headers=self.headers,
-                expected_status_code=http.client.FORBIDDEN
+                expected_status_code=http.client.FORBIDDEN,
             )
 
     def test_user_cannot_delete_endpoint_groups(self):
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
 
         with self.test_client() as c:
             c.delete(
-                '/v3/OS-EP-FILTER/endpoint_groups/%s' % endpoint_group['id'],
+                '/v3/OS-EP-FILTER/endpoint_groups/{}'.format(
+                    endpoint_group['id']
+                ),
                 headers=self.headers,
-                expected_status_code=http.client.FORBIDDEN
+                expected_status_code=http.client.FORBIDDEN,
             )
 
     def test_user_cannot_add_endpoint_group_to_project(self):
         project = PROVIDERS.resource_api.create_project(
-            uuid.uuid4().hex, unit.new_project_ref(
-                domain_id=CONF.identity.default_domain_id
-            )
+            uuid.uuid4().hex,
+            unit.new_project_ref(domain_id=CONF.identity.default_domain_id),
         )
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         with self.test_client() as c:
-            c.put('/v3/OS-EP-FILTER/endpoint_groups/%s/projects/%s'
-                  % (endpoint_group['id'], project['id']),
-                  headers=self.headers,
-                  expected_status_code=http.client.FORBIDDEN
-                  )
+            c.put(
+                '/v3/OS-EP-FILTER/endpoint_groups/{}/projects/{}'.format(
+                    endpoint_group['id'], project['id']
+                ),
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
+            )
 
     def test_cannot_remove_endpoint_group_from_project(self):
         project = PROVIDERS.resource_api.create_project(
-            uuid.uuid4().hex, unit.new_project_ref(
-                domain_id=CONF.identity.default_domain_id
-            )
+            uuid.uuid4().hex,
+            unit.new_project_ref(domain_id=CONF.identity.default_domain_id),
         )
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         with self.test_client() as c:
-            c.delete('/v3/OS-EP-FILTER/endpoint_groups/%s/projects/%s'
-                     % (endpoint_group['id'], project['id']),
-                     headers=self.headers,
-                     expected_status_code=http.client.FORBIDDEN
-                     )
+            c.delete(
+                '/v3/OS-EP-FILTER/endpoint_groups/{}/projects/{}'.format(
+                    endpoint_group['id'], project['id']
+                ),
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
+            )
 
 
-class SystemReaderTests(base_classes.TestCaseWithBootstrap,
-                        common_auth.AuthTestMixin,
-                        _SystemUserEndpointGroupsTests,
-                        _SystemReaderAndMemberUserEndpointGroupsTests):
-
+class SystemReaderTests(
+    base_classes.TestCaseWithBootstrap,
+    common_auth.AuthTestMixin,
+    _SystemUserEndpointGroupsTests,
+    _SystemReaderAndMemberUserEndpointGroupsTests,
+):
     def setUp(self):
-        super(SystemReaderTests, self).setUp()
+        super().setUp()
         self.loadapp()
         self.useFixture(ksfixtures.Policy(self.config_fixture))
         self.config_fixture.config(group='oslo_policy', enforce_scope=True)
@@ -423,16 +492,15 @@ class SystemReaderTests(base_classes.TestCaseWithBootstrap,
         system_reader = unit.new_user_ref(
             domain_id=CONF.identity.default_domain_id
         )
-        self.user_id = PROVIDERS.identity_api.create_user(
-            system_reader
-        )['id']
+        self.user_id = PROVIDERS.identity_api.create_user(system_reader)['id']
         PROVIDERS.assignment_api.create_system_grant_for_user(
             self.user_id, self.bootstrapper.reader_role_id
         )
 
         auth = self.build_authentication_request(
-            user_id=self.user_id, password=system_reader['password'],
-            system=True
+            user_id=self.user_id,
+            password=system_reader['password'],
+            system=True,
         )
 
         # Grab a token using the persona we're testing and prepare headers
@@ -443,13 +511,14 @@ class SystemReaderTests(base_classes.TestCaseWithBootstrap,
             self.headers = {'X-Auth-Token': self.token_id}
 
 
-class SystemMemberTests(base_classes.TestCaseWithBootstrap,
-                        common_auth.AuthTestMixin,
-                        _SystemUserEndpointGroupsTests,
-                        _SystemReaderAndMemberUserEndpointGroupsTests):
-
+class SystemMemberTests(
+    base_classes.TestCaseWithBootstrap,
+    common_auth.AuthTestMixin,
+    _SystemUserEndpointGroupsTests,
+    _SystemReaderAndMemberUserEndpointGroupsTests,
+):
     def setUp(self):
-        super(SystemMemberTests, self).setUp()
+        super().setUp()
         self.loadapp()
         self.useFixture(ksfixtures.Policy(self.config_fixture))
         self.config_fixture.config(group='oslo_policy', enforce_scope=True)
@@ -457,16 +526,15 @@ class SystemMemberTests(base_classes.TestCaseWithBootstrap,
         system_member = unit.new_user_ref(
             domain_id=CONF.identity.default_domain_id
         )
-        self.user_id = PROVIDERS.identity_api.create_user(
-            system_member
-        )['id']
+        self.user_id = PROVIDERS.identity_api.create_user(system_member)['id']
         PROVIDERS.assignment_api.create_system_grant_for_user(
             self.user_id, self.bootstrapper.member_role_id
         )
 
         auth = self.build_authentication_request(
-            user_id=self.user_id, password=system_member['password'],
-            system=True
+            user_id=self.user_id,
+            password=system_member['password'],
+            system=True,
         )
 
         # Grab a token using the persona we're testing and prepare headers
@@ -477,12 +545,13 @@ class SystemMemberTests(base_classes.TestCaseWithBootstrap,
             self.headers = {'X-Auth-Token': self.token_id}
 
 
-class SystemAdminTests(base_classes.TestCaseWithBootstrap,
-                       common_auth.AuthTestMixin,
-                       _SystemUserEndpointGroupsTests):
-
+class SystemAdminTests(
+    base_classes.TestCaseWithBootstrap,
+    common_auth.AuthTestMixin,
+    _SystemUserEndpointGroupsTests,
+):
     def setUp(self):
-        super(SystemAdminTests, self).setUp()
+        super().setUp()
         self.loadapp()
         self.useFixture(ksfixtures.Policy(self.config_fixture))
         self.config_fixture.config(group='oslo_policy', enforce_scope=True)
@@ -493,7 +562,7 @@ class SystemAdminTests(base_classes.TestCaseWithBootstrap,
         auth = self.build_authentication_request(
             user_id=self.user_id,
             password=self.bootstrapper.admin_password,
-            system=True
+            system=True,
         )
 
         # Grab a token using the persona we're testing and prepare headers
@@ -509,18 +578,21 @@ class SystemAdminTests(base_classes.TestCaseWithBootstrap,
                 'id': uuid.uuid4().hex,
                 'description': uuid.uuid4().hex,
                 'filters': {'interface': 'public'},
-                'name': uuid.uuid4().hex
+                'name': uuid.uuid4().hex,
             }
         }
 
         with self.test_client() as c:
             c.post(
-                '/v3/OS-EP-FILTER/endpoint_groups', json=create,
-                headers=self.headers)
+                '/v3/OS-EP-FILTER/endpoint_groups',
+                json=create,
+                headers=self.headers,
+            )
 
     def test_user_can_update_endpoint_group(self):
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
@@ -529,66 +601,78 @@ class SystemAdminTests(base_classes.TestCaseWithBootstrap,
 
         with self.test_client() as c:
             c.patch(
-                '/v3/OS-EP-FILTER/endpoint_groups/%s' % endpoint_group['id'],
+                '/v3/OS-EP-FILTER/endpoint_groups/{}'.format(
+                    endpoint_group['id']
+                ),
                 json=update,
-                headers=self.headers)
+                headers=self.headers,
+            )
 
     def test_user_can_delete_endpoint_group(self):
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
 
         with self.test_client() as c:
             c.delete(
-                '/v3/OS-EP-FILTER/endpoint_groups/%s' % endpoint_group['id'],
-                headers=self.headers
+                '/v3/OS-EP-FILTER/endpoint_groups/{}'.format(
+                    endpoint_group['id']
+                ),
+                headers=self.headers,
             )
 
     def test_user_add_endpoint_group_to_project(self):
         project = PROVIDERS.resource_api.create_project(
-            uuid.uuid4().hex, unit.new_project_ref(
-                domain_id=CONF.identity.default_domain_id
-            )
+            uuid.uuid4().hex,
+            unit.new_project_ref(domain_id=CONF.identity.default_domain_id),
         )
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         with self.test_client() as c:
-            c.put('/v3/OS-EP-FILTER/endpoint_groups/%s/projects/%s'
-                  % (endpoint_group['id'], project['id']),
-                  headers=self.headers
-                  )
+            c.put(
+                '/v3/OS-EP-FILTER/endpoint_groups/{}/projects/{}'.format(
+                    endpoint_group['id'], project['id']
+                ),
+                headers=self.headers,
+            )
 
     def test_remove_endpoint_group_from_project(self):
         project = PROVIDERS.resource_api.create_project(
-            uuid.uuid4().hex, unit.new_project_ref(
-                domain_id=CONF.identity.default_domain_id
-            )
+            uuid.uuid4().hex,
+            unit.new_project_ref(domain_id=CONF.identity.default_domain_id),
         )
         endpoint_group = unit.new_endpoint_group_ref(
-            filters={'interface': 'public'})
+            filters={'interface': 'public'}
+        )
         endpoint_group = PROVIDERS.catalog_api.create_endpoint_group(
             endpoint_group['id'], endpoint_group
         )
         PROVIDERS.catalog_api.add_endpoint_group_to_project(
-            endpoint_group['id'], project['id'])
+            endpoint_group['id'], project['id']
+        )
         with self.test_client() as c:
-            c.delete('/v3/OS-EP-FILTER/endpoint_groups/%s/projects/%s'
-                     % (endpoint_group['id'], project['id']),
-                     headers=self.headers
-                     )
+            c.delete(
+                '/v3/OS-EP-FILTER/endpoint_groups/{}/projects/{}'.format(
+                    endpoint_group['id'], project['id']
+                ),
+                headers=self.headers,
+            )
 
 
-class DomainUserTests(base_classes.TestCaseWithBootstrap,
-                      common_auth.AuthTestMixin,
-                      _DomainAndProjectUserEndpointGroupTests):
-
+class DomainUserTests(
+    base_classes.TestCaseWithBootstrap,
+    common_auth.AuthTestMixin,
+    _DomainAndProjectUserEndpointGroupTests,
+):
     def setUp(self):
-        super(DomainUserTests, self).setUp()
+        super().setUp()
         self.loadapp()
         self.useFixture(ksfixtures.Policy(self.config_fixture))
         self.config_fixture.config(group='oslo_policy', enforce_scope=True)
@@ -600,14 +684,15 @@ class DomainUserTests(base_classes.TestCaseWithBootstrap,
         domain_admin = unit.new_user_ref(domain_id=self.domain_id)
         self.user_id = PROVIDERS.identity_api.create_user(domain_admin)['id']
         PROVIDERS.assignment_api.create_grant(
-            self.bootstrapper.admin_role_id, user_id=self.user_id,
-            domain_id=self.domain_id
+            self.bootstrapper.admin_role_id,
+            user_id=self.user_id,
+            domain_id=self.domain_id,
         )
 
         auth = self.build_authentication_request(
             user_id=self.user_id,
             password=domain_admin['password'],
-            domain_id=self.domain_id
+            domain_id=self.domain_id,
         )
 
         # Grab a token using the persona we're testing and prepare headers
@@ -618,12 +703,13 @@ class DomainUserTests(base_classes.TestCaseWithBootstrap,
             self.headers = {'X-Auth-Token': self.token_id}
 
 
-class ProjectUserTests(base_classes.TestCaseWithBootstrap,
-                       common_auth.AuthTestMixin,
-                       _DomainAndProjectUserEndpointGroupTests):
-
+class ProjectUserTests(
+    base_classes.TestCaseWithBootstrap,
+    common_auth.AuthTestMixin,
+    _DomainAndProjectUserEndpointGroupTests,
+):
     def setUp(self):
-        super(ProjectUserTests, self).setUp()
+        super().setUp()
         self.loadapp()
         self.useFixture(ksfixtures.Policy(self.config_fixture))
         self.config_fixture.config(group='oslo_policy', enforce_scope=True)
@@ -632,7 +718,7 @@ class ProjectUserTests(base_classes.TestCaseWithBootstrap,
         auth = self.build_authentication_request(
             user_id=self.user_id,
             password=self.bootstrapper.admin_password,
-            project_id=self.bootstrapper.project_id
+            project_id=self.bootstrapper.project_id,
         )
 
         # Grab a token using the persona we're testing and prepare headers
@@ -644,12 +730,12 @@ class ProjectUserTests(base_classes.TestCaseWithBootstrap,
 
 
 class ProjectUserTestsWithoutEnforceScope(
-        base_classes.TestCaseWithBootstrap,
-        common_auth.AuthTestMixin,
-        _DomainAndProjectUserEndpointGroupTests):
-
+    base_classes.TestCaseWithBootstrap,
+    common_auth.AuthTestMixin,
+    _DomainAndProjectUserEndpointGroupTests,
+):
     def setUp(self):
-        super(ProjectUserTestsWithoutEnforceScope, self).setUp()
+        super().setUp()
         self.loadapp()
         self.useFixture(ksfixtures.Policy(self.config_fixture))
 
@@ -668,14 +754,15 @@ class ProjectUserTestsWithoutEnforceScope(
         )['id']
 
         PROVIDERS.assignment_api.create_grant(
-            self.bootstrapper.member_role_id, user_id=self.user_id,
-            project_id=self.project_id
+            self.bootstrapper.member_role_id,
+            user_id=self.user_id,
+            project_id=self.project_id,
         )
 
         auth = self.build_authentication_request(
             user_id=self.user_id,
             password=user['password'],
-            project_id=self.project_id
+            project_id=self.project_id,
         )
 
         # Grab a token using the persona we're testing and prepare headers
