@@ -10,9 +10,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import uuid
-
 import http.client
+import uuid
 
 from keystone.common import provider_api
 import keystone.conf
@@ -25,7 +24,7 @@ CONF = keystone.conf.CONF
 PROVIDERS = provider_api.ProviderAPIs
 
 
-class _SystemUserIdentityProviderTests(object):
+class _SystemUserIdentityProviderTests:
     """Common default functionality for all system users."""
 
     def test_user_can_list_identity_providers(self):
@@ -50,12 +49,12 @@ class _SystemUserIdentityProviderTests(object):
 
         with self.test_client() as c:
             c.get(
-                '/v3/OS-FEDERATION/identity_providers/%s' % idp['id'],
-                headers=self.headers
+                '/v3/OS-FEDERATION/identity_providers/{}'.format(idp['id']),
+                headers=self.headers,
             )
 
 
-class _SystemReaderAndMemberIdentityProviderTests(object):
+class _SystemReaderAndMemberIdentityProviderTests:
     """Common default functionality for system readers and system members."""
 
     def test_user_cannot_create_identity_providers(self):
@@ -63,9 +62,10 @@ class _SystemReaderAndMemberIdentityProviderTests(object):
 
         with self.test_client() as c:
             c.put(
-                '/v3/OS-FEDERATION/identity_providers/%s' % uuid.uuid4().hex,
-                json=create, headers=self.headers,
-                expected_status_code=http.client.FORBIDDEN
+                f'/v3/OS-FEDERATION/identity_providers/{uuid.uuid4().hex}',
+                json=create,
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
             )
 
     def test_user_cannot_update_identity_providers(self):
@@ -77,9 +77,10 @@ class _SystemReaderAndMemberIdentityProviderTests(object):
 
         with self.test_client() as c:
             c.patch(
-                '/v3/OS-FEDERATION/identity_providers/%s' % idp['id'],
-                json=update, headers=self.headers,
-                expected_status_code=http.client.FORBIDDEN
+                '/v3/OS-FEDERATION/identity_providers/{}'.format(idp['id']),
+                json=update,
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
             )
 
     def test_user_cannot_delete_identity_providers(self):
@@ -89,13 +90,13 @@ class _SystemReaderAndMemberIdentityProviderTests(object):
 
         with self.test_client() as c:
             c.delete(
-                '/v3/OS-FEDERATION/identity_providers/%s' % idp['id'],
+                '/v3/OS-FEDERATION/identity_providers/{}'.format(idp['id']),
                 headers=self.headers,
-                expected_status_code=http.client.FORBIDDEN
+                expected_status_code=http.client.FORBIDDEN,
             )
 
 
-class _DomainAndProjectUserIdentityProviderTests(object):
+class _DomainAndProjectUserIdentityProviderTests:
     """Common functionality for all domain and project users."""
 
     def test_user_cannot_create_identity_providers(self):
@@ -103,9 +104,10 @@ class _DomainAndProjectUserIdentityProviderTests(object):
 
         with self.test_client() as c:
             c.put(
-                '/v3/OS-FEDERATION/identity_providers/%s' % uuid.uuid4().hex,
-                json=create, headers=self.headers,
-                expected_status_code=http.client.FORBIDDEN
+                f'/v3/OS-FEDERATION/identity_providers/{uuid.uuid4().hex}',
+                json=create,
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
             )
 
     def test_user_cannot_update_identity_providers(self):
@@ -117,9 +119,10 @@ class _DomainAndProjectUserIdentityProviderTests(object):
 
         with self.test_client() as c:
             c.patch(
-                '/v3/OS-FEDERATION/identity_providers/%s' % idp['id'],
-                json=update, headers=self.headers,
-                expected_status_code=http.client.FORBIDDEN
+                '/v3/OS-FEDERATION/identity_providers/{}'.format(idp['id']),
+                json=update,
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
             )
 
     def test_user_cannot_list_identity_providers(self):
@@ -129,8 +132,9 @@ class _DomainAndProjectUserIdentityProviderTests(object):
 
         with self.test_client() as c:
             c.get(
-                '/v3/OS-FEDERATION/identity_providers', headers=self.headers,
-                expected_status_code=http.client.FORBIDDEN
+                '/v3/OS-FEDERATION/identity_providers',
+                headers=self.headers,
+                expected_status_code=http.client.FORBIDDEN,
             )
 
     def test_user_cannot_get_an_identity_provider(self):
@@ -140,9 +144,9 @@ class _DomainAndProjectUserIdentityProviderTests(object):
 
         with self.test_client() as c:
             c.get(
-                '/v3/OS-FEDERATION/identity_providers/%s' % idp['id'],
+                '/v3/OS-FEDERATION/identity_providers/{}'.format(idp['id']),
                 headers=self.headers,
-                expected_status_code=http.client.FORBIDDEN
+                expected_status_code=http.client.FORBIDDEN,
             )
 
     def test_user_cannot_delete_identity_providers(self):
@@ -152,19 +156,20 @@ class _DomainAndProjectUserIdentityProviderTests(object):
 
         with self.test_client() as c:
             c.delete(
-                '/v3/OS-FEDERATION/identity_providers/%s' % idp['id'],
+                '/v3/OS-FEDERATION/identity_providers/{}'.format(idp['id']),
                 headers=self.headers,
-                expected_status_code=http.client.FORBIDDEN
+                expected_status_code=http.client.FORBIDDEN,
             )
 
 
-class SystemReaderTests(base_classes.TestCaseWithBootstrap,
-                        common_auth.AuthTestMixin,
-                        _SystemUserIdentityProviderTests,
-                        _SystemReaderAndMemberIdentityProviderTests):
-
+class SystemReaderTests(
+    base_classes.TestCaseWithBootstrap,
+    common_auth.AuthTestMixin,
+    _SystemUserIdentityProviderTests,
+    _SystemReaderAndMemberIdentityProviderTests,
+):
     def setUp(self):
-        super(SystemReaderTests, self).setUp()
+        super().setUp()
         self.loadapp()
         self.useFixture(ksfixtures.Policy(self.config_fixture))
         self.config_fixture.config(group='oslo_policy', enforce_scope=True)
@@ -172,16 +177,15 @@ class SystemReaderTests(base_classes.TestCaseWithBootstrap,
         system_reader = unit.new_user_ref(
             domain_id=CONF.identity.default_domain_id
         )
-        self.user_id = PROVIDERS.identity_api.create_user(
-            system_reader
-        )['id']
+        self.user_id = PROVIDERS.identity_api.create_user(system_reader)['id']
         PROVIDERS.assignment_api.create_system_grant_for_user(
             self.user_id, self.bootstrapper.reader_role_id
         )
 
         auth = self.build_authentication_request(
-            user_id=self.user_id, password=system_reader['password'],
-            system=True
+            user_id=self.user_id,
+            password=system_reader['password'],
+            system=True,
         )
 
         # Grab a token using the persona we're testing and prepare headers
@@ -192,13 +196,14 @@ class SystemReaderTests(base_classes.TestCaseWithBootstrap,
             self.headers = {'X-Auth-Token': self.token_id}
 
 
-class SystemMemberTests(base_classes.TestCaseWithBootstrap,
-                        common_auth.AuthTestMixin,
-                        _SystemUserIdentityProviderTests,
-                        _SystemReaderAndMemberIdentityProviderTests):
-
+class SystemMemberTests(
+    base_classes.TestCaseWithBootstrap,
+    common_auth.AuthTestMixin,
+    _SystemUserIdentityProviderTests,
+    _SystemReaderAndMemberIdentityProviderTests,
+):
     def setUp(self):
-        super(SystemMemberTests, self).setUp()
+        super().setUp()
         self.loadapp()
         self.useFixture(ksfixtures.Policy(self.config_fixture))
         self.config_fixture.config(group='oslo_policy', enforce_scope=True)
@@ -206,16 +211,15 @@ class SystemMemberTests(base_classes.TestCaseWithBootstrap,
         system_member = unit.new_user_ref(
             domain_id=CONF.identity.default_domain_id
         )
-        self.user_id = PROVIDERS.identity_api.create_user(
-            system_member
-        )['id']
+        self.user_id = PROVIDERS.identity_api.create_user(system_member)['id']
         PROVIDERS.assignment_api.create_system_grant_for_user(
             self.user_id, self.bootstrapper.member_role_id
         )
 
         auth = self.build_authentication_request(
-            user_id=self.user_id, password=system_member['password'],
-            system=True
+            user_id=self.user_id,
+            password=system_member['password'],
+            system=True,
         )
 
         # Grab a token using the persona we're testing and prepare headers
@@ -226,12 +230,13 @@ class SystemMemberTests(base_classes.TestCaseWithBootstrap,
             self.headers = {'X-Auth-Token': self.token_id}
 
 
-class SystemAdminTests(base_classes.TestCaseWithBootstrap,
-                       common_auth.AuthTestMixin,
-                       _SystemUserIdentityProviderTests):
-
+class SystemAdminTests(
+    base_classes.TestCaseWithBootstrap,
+    common_auth.AuthTestMixin,
+    _SystemUserIdentityProviderTests,
+):
     def setUp(self):
-        super(SystemAdminTests, self).setUp()
+        super().setUp()
         self.loadapp()
         self.useFixture(ksfixtures.Policy(self.config_fixture))
         self.config_fixture.config(group='oslo_policy', enforce_scope=True)
@@ -242,7 +247,7 @@ class SystemAdminTests(base_classes.TestCaseWithBootstrap,
         auth = self.build_authentication_request(
             user_id=self.user_id,
             password=self.bootstrapper.admin_password,
-            system=True
+            system=True,
         )
 
         # Grab a token using the persona we're testing and prepare headers
@@ -257,9 +262,10 @@ class SystemAdminTests(base_classes.TestCaseWithBootstrap,
 
         with self.test_client() as c:
             c.put(
-                '/v3/OS-FEDERATION/identity_providers/%s' % uuid.uuid4().hex,
-                json=create, headers=self.headers,
-                expected_status_code=http.client.CREATED
+                f'/v3/OS-FEDERATION/identity_providers/{uuid.uuid4().hex}',
+                json=create,
+                headers=self.headers,
+                expected_status_code=http.client.CREATED,
             )
 
     def test_user_can_update_identity_providers(self):
@@ -271,8 +277,9 @@ class SystemAdminTests(base_classes.TestCaseWithBootstrap,
 
         with self.test_client() as c:
             c.patch(
-                '/v3/OS-FEDERATION/identity_providers/%s' % idp['id'],
-                json=update, headers=self.headers
+                '/v3/OS-FEDERATION/identity_providers/{}'.format(idp['id']),
+                json=update,
+                headers=self.headers,
             )
 
     def test_user_can_delete_identity_providers(self):
@@ -282,17 +289,18 @@ class SystemAdminTests(base_classes.TestCaseWithBootstrap,
 
         with self.test_client() as c:
             c.delete(
-                '/v3/OS-FEDERATION/identity_providers/%s' % idp['id'],
-                headers=self.headers
+                '/v3/OS-FEDERATION/identity_providers/{}'.format(idp['id']),
+                headers=self.headers,
             )
 
 
-class DomainUserTests(base_classes.TestCaseWithBootstrap,
-                      common_auth.AuthTestMixin,
-                      _DomainAndProjectUserIdentityProviderTests):
-
+class DomainUserTests(
+    base_classes.TestCaseWithBootstrap,
+    common_auth.AuthTestMixin,
+    _DomainAndProjectUserIdentityProviderTests,
+):
     def setUp(self):
-        super(DomainUserTests, self).setUp()
+        super().setUp()
         self.loadapp()
         self.useFixture(ksfixtures.Policy(self.config_fixture))
         self.config_fixture.config(group='oslo_policy', enforce_scope=True)
@@ -304,14 +312,15 @@ class DomainUserTests(base_classes.TestCaseWithBootstrap,
         domain_admin = unit.new_user_ref(domain_id=self.domain_id)
         self.user_id = PROVIDERS.identity_api.create_user(domain_admin)['id']
         PROVIDERS.assignment_api.create_grant(
-            self.bootstrapper.admin_role_id, user_id=self.user_id,
-            domain_id=self.domain_id
+            self.bootstrapper.admin_role_id,
+            user_id=self.user_id,
+            domain_id=self.domain_id,
         )
 
         auth = self.build_authentication_request(
             user_id=self.user_id,
             password=domain_admin['password'],
-            domain_id=self.domain_id
+            domain_id=self.domain_id,
         )
 
         # Grab a token using the persona we're testing and prepare headers
@@ -322,12 +331,13 @@ class DomainUserTests(base_classes.TestCaseWithBootstrap,
             self.headers = {'X-Auth-Token': self.token_id}
 
 
-class ProjectUserTests(base_classes.TestCaseWithBootstrap,
-                       common_auth.AuthTestMixin,
-                       _DomainAndProjectUserIdentityProviderTests):
-
+class ProjectUserTests(
+    base_classes.TestCaseWithBootstrap,
+    common_auth.AuthTestMixin,
+    _DomainAndProjectUserIdentityProviderTests,
+):
     def setUp(self):
-        super(ProjectUserTests, self).setUp()
+        super().setUp()
         self.loadapp()
         self.useFixture(ksfixtures.Policy(self.config_fixture))
         self.config_fixture.config(group='oslo_policy', enforce_scope=True)
@@ -336,7 +346,7 @@ class ProjectUserTests(base_classes.TestCaseWithBootstrap,
         auth = self.build_authentication_request(
             user_id=self.user_id,
             password=self.bootstrapper.admin_password,
-            project_id=self.bootstrapper.project_id
+            project_id=self.bootstrapper.project_id,
         )
 
         # Grab a token using the persona we're testing and prepare headers
@@ -348,12 +358,12 @@ class ProjectUserTests(base_classes.TestCaseWithBootstrap,
 
 
 class ProjectUserTestsWithoutEnforceScope(
-        base_classes.TestCaseWithBootstrap,
-        common_auth.AuthTestMixin,
-        _DomainAndProjectUserIdentityProviderTests):
-
+    base_classes.TestCaseWithBootstrap,
+    common_auth.AuthTestMixin,
+    _DomainAndProjectUserIdentityProviderTests,
+):
     def setUp(self):
-        super(ProjectUserTestsWithoutEnforceScope, self).setUp()
+        super().setUp()
         self.loadapp()
         self.useFixture(ksfixtures.Policy(self.config_fixture))
 
@@ -372,14 +382,15 @@ class ProjectUserTestsWithoutEnforceScope(
         )['id']
 
         PROVIDERS.assignment_api.create_grant(
-            self.bootstrapper.member_role_id, user_id=self.user_id,
-            project_id=self.project_id
+            self.bootstrapper.member_role_id,
+            user_id=self.user_id,
+            project_id=self.project_id,
         )
 
         auth = self.build_authentication_request(
             user_id=self.user_id,
             password=user['password'],
-            project_id=self.project_id
+            project_id=self.project_id,
         )
 
         # Grab a token using the persona we're testing and prepare headers
