@@ -18,20 +18,24 @@ import contextlib
 import datetime
 import functools
 import hashlib
+import http.client
 import json
-import secrets
-
-import ldap
 import os
+import secrets
 import shutil
 import socket
 import sys
+import unittest
 import uuid
 
+from cryptography.hazmat.primitives.asymmetric import rsa
+from cryptography.hazmat.primitives import hashes
+from cryptography.hazmat.primitives.serialization import Encoding
+from cryptography import x509
 import fixtures
 import flask
 from flask import testing as flask_testing
-import http.client
+import ldap
 from oslo_config import fixture as config_fixture
 from oslo_context import context as oslo_context
 from oslo_context import fixture as oslo_ctx_fixture
@@ -39,9 +43,7 @@ from oslo_log import fixture as log_fixture
 from oslo_log import log
 from oslo_utils import timeutils
 import testtools
-from testtools import testcase
 
-import keystone.api
 from keystone.common import context
 from keystone.common import json_home
 from keystone.common import provider_api
@@ -55,7 +57,6 @@ from keystone.server.flask import application as flask_app
 from keystone.server.flask import core as keystone_flask
 from keystone.tests.unit import ksfixtures
 
-
 keystone.conf.configure()
 keystone.conf.set_config_defaults()
 
@@ -92,7 +93,7 @@ os.makedirs(TMPDIR)
 atexit.register(shutil.rmtree, TMPDIR)
 
 
-class dirs(object):
+class dirs:
     @staticmethod
     def root(*p):
         return os.path.join(ROOTDIR, *p)
@@ -132,8 +133,7 @@ def skip_if_cache_disabled(*sections):
     In the code fragment::
 
         @skip_if_cache_is_disabled('assignment', 'token')
-        def test_method(*args):
-            ...
+        def test_method(*args): ...
 
     The method test_method would be skipped if caching is disabled globally via
     the `enabled` option in the `cache` section of the configuration or if
@@ -146,18 +146,21 @@ def skip_if_cache_disabled(*sections):
     section of the configuration is true.
 
     """
+
     def wrapper(f):
         @functools.wraps(f)
         def inner(*args, **kwargs):
             if not CONF.cache.enabled:
-                raise testcase.TestSkipped('Cache globally disabled.')
+                raise unittest.SkipTest('Cache globally disabled.')
             for s in sections:
                 conf_sec = getattr(CONF, s, None)
                 if conf_sec is not None:
                     if not getattr(conf_sec, 'caching', True):
-                        raise testcase.TestSkipped('%s caching disabled.' % s)
+                        raise unittest.SkipTest(f'{s} caching disabled.')
             return f(*args, **kwargs)
+
         return inner
+
     return wrapper
 
 
@@ -170,21 +173,24 @@ def skip_if_cache_is_enabled(*sections):
                     conf_sec = getattr(CONF, s, None)
                     if conf_sec is not None:
                         if getattr(conf_sec, 'caching', True):
-                            raise testcase.TestSkipped('%s caching enabled.' %
-                                                       s)
+                            raise unittest.SkipTest(f'{s} caching enabled.')
             return f(*args, **kwargs)
+
         return inner
+
     return wrapper
 
 
 def skip_if_no_multiple_domains_support(f):
     """Decorator to skip tests for identity drivers limited to one domain."""
+
     @functools.wraps(f)
     def wrapper(*args, **kwargs):
         test_obj = args[0]
         if not test_obj.identity_api.multiple_domains_supported:
-            raise testcase.TestSkipped('No multiple domains support')
+            raise unittest.SkipTest('No multiple domains support')
         return f(*args, **kwargs)
+
     return wrapper
 
 
@@ -196,7 +202,8 @@ def new_region_ref(parent_region_id=None, **kwargs):
     ref = {
         'id': uuid.uuid4().hex,
         'description': uuid.uuid4().hex,
-        'parent_region_id': parent_region_id}
+        'parent_region_id': parent_region_id,
+    }
 
     ref.update(kwargs)
     return ref
@@ -217,9 +224,9 @@ def new_service_ref(**kwargs):
 NEEDS_REGION_ID = object()
 
 
-def new_endpoint_ref(service_id, interface='public',
-                     region_id=NEEDS_REGION_ID, **kwargs):
-
+def new_endpoint_ref(
+    service_id, interface='public', region_id=NEEDS_REGION_ID, **kwargs
+):
     ref = {
         'id': uuid.uuid4().hex,
         'name': uuid.uuid4().hex,
@@ -245,20 +252,22 @@ def new_endpoint_group_ref(filters, **kwargs):
         'id': uuid.uuid4().hex,
         'description': uuid.uuid4().hex,
         'filters': filters,
-        'name': uuid.uuid4().hex
+        'name': uuid.uuid4().hex,
     }
     ref.update(kwargs)
     return ref
 
 
-def new_endpoint_ref_with_region(service_id, region, interface='public',
-                                 **kwargs):
+def new_endpoint_ref_with_region(
+    service_id, region, interface='public', **kwargs
+):
     """Define an endpoint_ref having a pre-3.2 form.
 
     Contains the deprecated 'region' instead of 'region_id'.
     """
-    ref = new_endpoint_ref(service_id, interface, region=region,
-                           region_id='invalid', **kwargs)
+    ref = new_endpoint_ref(
+        service_id, interface, region=region, region_id='invalid', **kwargs
+    )
     del ref['region_id']
     return ref
 
@@ -270,7 +279,7 @@ def new_domain_ref(**kwargs):
         'description': uuid.uuid4().hex,
         'enabled': True,
         'tags': [],
-        'options': {}
+        'options': {},
     }
     ref.update(kwargs)
     return ref
@@ -285,7 +294,7 @@ def new_project_ref(domain_id=None, is_domain=False, **kwargs):
         'domain_id': domain_id,
         'is_domain': is_domain,
         'tags': [],
-        'options': {}
+        'options': {},
     }
     # NOTE(henry-nash): We don't include parent_id in the initial list above
     # since specifying it is optional depending on where the project sits in
@@ -322,10 +331,7 @@ def new_federated_user_ref(idp_id=None, protocol_id=None, **kwargs):
 
 
 def new_mapping_ref(mapping_id=None, rules=None, **kwargs):
-    ref = {
-        'id': mapping_id or uuid.uuid4().hex,
-        'rules': rules or []
-    }
+    ref = {'id': mapping_id or uuid.uuid4().hex, 'rules': rules or []}
     ref.update(kwargs)
     return ref
 
@@ -334,18 +340,14 @@ def new_protocol_ref(protocol_id=None, idp_id=None, mapping_id=None, **kwargs):
     ref = {
         'id': protocol_id or 'saml2',
         'idp_id': idp_id or 'ORG_IDP',
-        'mapping_id': mapping_id or uuid.uuid4().hex
+        'mapping_id': mapping_id or uuid.uuid4().hex,
     }
     ref.update(kwargs)
     return ref
 
 
 def new_identity_provider_ref(idp_id=None, **kwargs):
-    ref = {
-        'id': idp_id or 'ORG_IDP',
-        'enabled': True,
-        'description': '',
-    }
+    ref = {'id': idp_id or 'ORG_IDP', 'enabled': True, 'description': ''}
     ref.update(kwargs)
     return ref
 
@@ -356,7 +358,7 @@ def new_service_provider_ref(**kwargs):
         'enabled': True,
         'description': uuid.uuid4().hex,
         'sp_url': 'https://' + uuid.uuid4().hex + '.com',
-        'relay_state_prefix': CONF.saml.relay_state_prefix
+        'relay_state_prefix': CONF.saml.relay_state_prefix,
     }
     ref.update(kwargs)
     return ref
@@ -367,18 +369,14 @@ def new_group_ref(domain_id, **kwargs):
         'id': uuid.uuid4().hex,
         'name': uuid.uuid4().hex,
         'description': uuid.uuid4().hex,
-        'domain_id': domain_id
+        'domain_id': domain_id,
     }
     ref.update(kwargs)
     return ref
 
 
 def new_credential_ref(user_id, project_id=None, type='cert', **kwargs):
-    ref = {
-        'id': uuid.uuid4().hex,
-        'user_id': user_id,
-        'type': type,
-    }
+    ref = {'id': uuid.uuid4().hex, 'user_id': user_id, 'type': type}
 
     if project_id:
         ref['project_id'] = project_id
@@ -393,11 +391,13 @@ def new_cert_credential(user_id, project_id=None, blob=None, **kwargs):
     if blob is None:
         blob = {'access': uuid.uuid4().hex, 'secret': uuid.uuid4().hex}
 
-    credential = new_credential_ref(user_id=user_id,
-                                    project_id=project_id,
-                                    blob=json.dumps(blob),
-                                    type='cert',
-                                    **kwargs)
+    credential = new_credential_ref(
+        user_id=user_id,
+        project_id=project_id,
+        blob=json.dumps(blob),
+        type='cert',
+        **kwargs,
+    )
     return blob, credential
 
 
@@ -406,18 +406,20 @@ def new_ec2_credential(user_id, project_id=None, blob=None, **kwargs):
         blob = {
             'access': uuid.uuid4().hex,
             'secret': uuid.uuid4().hex,
-            'trust_id': None
+            'trust_id': None,
         }
 
     if 'id' not in kwargs:
         access = blob['access'].encode('utf-8')
         kwargs['id'] = hashlib.sha256(access).hexdigest()
 
-    credential = new_credential_ref(user_id=user_id,
-                                    project_id=project_id,
-                                    blob=json.dumps(blob),
-                                    type='ec2',
-                                    **kwargs)
+    credential = new_credential_ref(
+        user_id=user_id,
+        project_id=project_id,
+        blob=json.dumps(blob),
+        type='ec2',
+        **kwargs,
+    )
     return blob, credential
 
 
@@ -426,17 +428,85 @@ def new_totp_credential(user_id, project_id=None, blob=None):
         # NOTE(notmorgan): 20 bytes of data from secrets.token_bytes for
         # a totp secret.
         blob = base64.b32encode(secrets.token_bytes(20)).decode('utf-8')
-    credential = new_credential_ref(user_id=user_id,
-                                    project_id=project_id,
-                                    blob=blob,
-                                    type='totp')
+    credential = new_credential_ref(
+        user_id=user_id, project_id=project_id, blob=blob, type='totp'
+    )
     return credential
 
 
-def new_application_credential_ref(roles=None,
-                                   name=None,
-                                   expires=None,
-                                   secret=None):
+def create_dn(
+    common_name=None,
+    locality_name=None,
+    state_or_province_name=None,
+    organization_name=None,
+    organizational_unit_name=None,
+    country_name=None,
+    street_address=None,
+    domain_component=None,
+    user_id=None,
+    email_address=None,
+):
+    oid = x509.NameOID
+    attr = x509.NameAttribute
+    dn = []
+    if common_name:
+        dn.append(attr(oid.COMMON_NAME, common_name))
+    if locality_name:
+        dn.append(attr(oid.LOCALITY_NAME, locality_name))
+    if state_or_province_name:
+        dn.append(attr(oid.STATE_OR_PROVINCE_NAME, state_or_province_name))
+    if organization_name:
+        dn.append(attr(oid.ORGANIZATION_NAME, organization_name))
+    if organizational_unit_name:
+        dn.append(attr(oid.ORGANIZATIONAL_UNIT_NAME, organizational_unit_name))
+    if country_name:
+        dn.append(attr(oid.COUNTRY_NAME, country_name))
+    if street_address:
+        dn.append(attr(oid.STREET_ADDRESS, street_address))
+    if domain_component:
+        dn.append(attr(oid.DOMAIN_COMPONENT, domain_component))
+    if user_id:
+        dn.append(attr(oid.USER_ID, user_id))
+    if email_address:
+        dn.append(attr(oid.EMAIL_ADDRESS, email_address))
+    return x509.Name(dn)
+
+
+def update_dn(dn1, dn2):
+    dn1_attrs = {attr.oid: attr for attr in dn1}
+    dn2_attrs = {attr.oid: attr for attr in dn2}
+    dn1_attrs.update(dn2_attrs)
+    return x509.Name(list(dn1_attrs.values()))
+
+
+def create_certificate(subject_dn, ca=None, ca_key=None):
+    private_key = rsa.generate_private_key(
+        public_exponent=65537, key_size=2048
+    )
+    issuer = ca.subject if ca else subject_dn
+    if not ca_key:
+        ca_key = private_key
+    today = datetime.datetime.today()
+    cert = x509.CertificateBuilder(
+        issuer_name=issuer,
+        subject_name=subject_dn,
+        public_key=private_key.public_key(),
+        serial_number=x509.random_serial_number(),
+        not_valid_before=today,
+        not_valid_after=today + datetime.timedelta(365, 0, 0),
+    ).sign(ca_key, hashes.SHA256())
+
+    return cert, private_key
+
+
+def create_pem_certificate(subject_dn, ca=None, ca_key=None):
+    cert, _ = create_certificate(subject_dn, ca=ca, ca_key=ca_key)
+    return cert.public_bytes(Encoding.PEM).decode('ascii')
+
+
+def new_application_credential_ref(
+    roles=None, name=None, expires=None, secret=None
+):
     ref = {
         'id': uuid.uuid4().hex,
         'name': uuid.uuid4().hex,
@@ -480,7 +550,7 @@ def new_policy_ref(**kwargs):
         'description': uuid.uuid4().hex,
         'enabled': True,
         # Store serialized JSON data as the blob to mimic real world usage.
-        'blob': json.dumps({'data': uuid.uuid4().hex, }),
+        'blob': json.dumps({'data': uuid.uuid4().hex}),
         'type': uuid.uuid4().hex,
     }
 
@@ -490,22 +560,29 @@ def new_policy_ref(**kwargs):
 
 def new_domain_config_ref(**kwargs):
     ref = {
-        "identity": {
-            "driver": "ldap"
-        },
+        "identity": {"driver": "ldap"},
         "ldap": {
             "url": "ldap://myldap.com:389/",
-            "user_tree_dn": "ou=Users,dc=my_new_root,dc=org"
-        }
+            "user_tree_dn": "ou=Users,dc=my_new_root,dc=org",
+        },
     }
     ref.update(kwargs)
     return ref
 
 
-def new_trust_ref(trustor_user_id, trustee_user_id, project_id=None,
-                  impersonation=None, expires=None, role_ids=None,
-                  role_names=None, remaining_uses=None,
-                  allow_redelegation=False, redelegation_count=None, **kwargs):
+def new_trust_ref(
+    trustor_user_id,
+    trustee_user_id,
+    project_id=None,
+    impersonation=None,
+    expires=None,
+    role_ids=None,
+    role_names=None,
+    remaining_uses=None,
+    allow_redelegation=False,
+    redelegation_count=None,
+    **kwargs,
+):
     ref = {
         'id': uuid.uuid4().hex,
         'trustor_user_id': trustor_user_id,
@@ -548,7 +625,7 @@ def new_registered_limit_ref(**kwargs):
         'service_id': uuid.uuid4().hex,
         'resource_name': uuid.uuid4().hex,
         'default_limit': 10,
-        'description': uuid.uuid4().hex
+        'description': uuid.uuid4().hex,
     }
 
     ref.update(kwargs)
@@ -560,7 +637,7 @@ def new_limit_ref(**kwargs):
         'service_id': uuid.uuid4().hex,
         'resource_name': uuid.uuid4().hex,
         'resource_limit': 10,
-        'description': uuid.uuid4().hex
+        'description': uuid.uuid4().hex,
     }
 
     ref.update(kwargs)
@@ -605,7 +682,9 @@ def _assert_expected_status(f):
         expected_status_code = kwargs.pop(
             'expected_status_code',
             _default_expected_responses.get(
-                f.__name__.lower(), http.client.OK))
+                f.__name__.lower(), http.client.OK
+            ),
+        )
         response = f(*args, **kwargs)
 
         # Logic to verify the response object is sane. Expand as needed
@@ -614,18 +693,17 @@ def _assert_expected_status(f):
             # an un-routed HTTP call was made. This allows us to avoid
             # misinterpreting HTTP 404 from Flask and HTTP 404 from a
             # resource that is not found (e.g. USER NOT FOUND) programmatically
-            raise AssertionError("I AM A TEAPOT(418): %s" % response.data)
+            raise AssertionError(f"I AM A TEAPOT(418): {response.data}")
 
         if response.status_code != expected_status_code:
             raise AssertionError(
                 'Expected HTTP Status does not match observed HTTP '
-                'Status: %(expected)s != %(observed)s (%(data)s)' % {
-                    'expected': expected_status_code,
-                    'observed': response.status_code,
-                    'data': response.data})
+                f'Status: {expected_status_code} != {response.status_code} ({response.data})'
+            )
 
         # return the original response object
         return response
+
     return inner
 
 
@@ -638,27 +716,27 @@ class KeystoneFlaskTestClient(flask_testing.FlaskClient):
 
     @_assert_expected_status
     def get(self, *args, **kwargs):
-        return super(KeystoneFlaskTestClient, self).get(*args, **kwargs)
+        return super().get(*args, **kwargs)
 
     @_assert_expected_status
     def head(self, *args, **kwargs):
-        return super(KeystoneFlaskTestClient, self).head(*args, **kwargs)
+        return super().head(*args, **kwargs)
 
     @_assert_expected_status
     def post(self, *args, **kwargs):
-        return super(KeystoneFlaskTestClient, self).post(*args, **kwargs)
+        return super().post(*args, **kwargs)
 
     @_assert_expected_status
     def patch(self, *args, **kwargs):
-        return super(KeystoneFlaskTestClient, self).patch(*args, **kwargs)
+        return super().patch(*args, **kwargs)
 
     @_assert_expected_status
     def put(self, *args, **kwargs):
-        return super(KeystoneFlaskTestClient, self).put(*args, **kwargs)
+        return super().put(*args, **kwargs)
 
     @_assert_expected_status
     def delete(self, *args, **kwargs):
-        return super(KeystoneFlaskTestClient, self).delete(*args, **kwargs)
+        return super().delete(*args, **kwargs)
 
 
 class BaseTestCase(testtools.TestCase):
@@ -671,13 +749,14 @@ class BaseTestCase(testtools.TestCase):
     """
 
     def setUp(self):
-        super(BaseTestCase, self).setUp()
+        super().setUp()
 
         self.useFixture(fixtures.NestedTempfile())
         self.useFixture(fixtures.TempHomeDir())
 
-        self.useFixture(fixtures.MockPatchObject(sys, 'exit',
-                                                 side_effect=UnexpectedExit))
+        self.useFixture(
+            fixtures.MockPatchObject(sys, 'exit', side_effect=UnexpectedExit)
+        )
         self.useFixture(log_fixture.get_logging_handle_error_fixture())
         self.stdlog = self.useFixture(ksfixtures.StandardLogging())
         self.useFixture(ksfixtures.WarningsFixture())
@@ -688,22 +767,26 @@ class BaseTestCase(testtools.TestCase):
         self.useFixture(oslo_ctx_fixture.ClearRequestContext())
 
         orig_debug_level = ldap.get_option(ldap.OPT_DEBUG_LEVEL)
-        self.addCleanup(ldap.set_option, ldap.OPT_DEBUG_LEVEL,
-                        orig_debug_level)
+        self.addCleanup(
+            ldap.set_option, ldap.OPT_DEBUG_LEVEL, orig_debug_level
+        )
         orig_tls_cacertfile = ldap.get_option(ldap.OPT_X_TLS_CACERTFILE)
         if orig_tls_cacertfile is None:
             orig_tls_cacertfile = ''
-        self.addCleanup(ldap.set_option, ldap.OPT_X_TLS_CACERTFILE,
-                        orig_tls_cacertfile)
+        self.addCleanup(
+            ldap.set_option, ldap.OPT_X_TLS_CACERTFILE, orig_tls_cacertfile
+        )
         orig_tls_cacertdir = ldap.get_option(ldap.OPT_X_TLS_CACERTDIR)
         # Setting orig_tls_cacertdir to None is not allowed.
         if orig_tls_cacertdir is None:
             orig_tls_cacertdir = ''
-        self.addCleanup(ldap.set_option, ldap.OPT_X_TLS_CACERTDIR,
-                        orig_tls_cacertdir)
+        self.addCleanup(
+            ldap.set_option, ldap.OPT_X_TLS_CACERTDIR, orig_tls_cacertdir
+        )
         orig_tls_require_cert = ldap.get_option(ldap.OPT_X_TLS_REQUIRE_CERT)
-        self.addCleanup(ldap.set_option, ldap.OPT_X_TLS_REQUIRE_CERT,
-                        orig_tls_require_cert)
+        self.addCleanup(
+            ldap.set_option, ldap.OPT_X_TLS_REQUIRE_CERT, orig_tls_require_cert
+        )
         self.addCleanup(ks_ldap.PooledLDAPHandler.connection_pools.clear)
 
     def cleanup_instance(self, *names):
@@ -712,6 +795,7 @@ class BaseTestCase(testtools.TestCase):
         :returns: a callable that uses a closure to delete instance attributes
 
         """
+
         def cleanup():
             for name in names:
                 # TODO(dstanek): remove this 'if' statement once
@@ -719,17 +803,19 @@ class BaseTestCase(testtools.TestCase):
                 # per test
                 if hasattr(self, name):
                     delattr(self, name)
+
         return cleanup
 
     def skip_if_env_not_set(self, env_var):
         if not os.environ.get(env_var):
-            self.skipTest('Env variable %s is not set.' % env_var)
+            self.skipTest(f'Env variable {env_var} is not set.')
 
     def skip_test_overrides(self, *args, **kwargs):
         if self._check_for_method_in_parents(self._testMethodName):
-            return super(BaseTestCase, self).skipTest(*args, **kwargs)
-        raise Exception('%r is not a previously defined test method'
-                        % self._testMethodName)
+            return super().skipTest(*args, **kwargs)
+        raise Exception(
+            f'{self._testMethodName!r} is not a previously defined test method'
+        )
 
     def _check_for_method_in_parents(self, name):
         # skip first to get to parents
@@ -747,10 +833,10 @@ class BaseTestCase(testtools.TestCase):
         # is a hard error and should not pass testing.
         def page_not_found_teapot(e):
             content = (
-                'TEST PROGRAMMING ERROR - Reached a 404 from an unrouted (`%s`'
+                f'TEST PROGRAMMING ERROR - Reached a 404 from an unrouted (`{flask.request.url}`'
                 ') path. Be sure the test is requesting the right resource '
-                'and that all blueprints are registered with the flask app.' %
-                flask.request.url)
+                'and that all blueprints are registered with the flask app.'
+            )
             return content, 418
 
         app.register_error_handler(404, page_not_found_teapot)
@@ -763,7 +849,6 @@ class BaseTestCase(testtools.TestCase):
 
 
 class TestCase(BaseTestCase):
-
     def config_files(self):
         return []
 
@@ -780,12 +865,13 @@ class TestCase(BaseTestCase):
         query_string = kwargs.pop('query_string', None)
         if query_string:
             # Make sure query string is properly added to the context
-            path = '{path}?{qs}'.format(path=path, qs=query_string)
+            path = f'{path}?{query_string}'
 
         if not environ.get(context.REQUEST_CONTEXT_ENV):
             environ[context.REQUEST_CONTEXT_ENV] = context.RequestContext(
                 is_admin=is_admin,
-                authenticated=kwargs.pop('authenticated', True))
+                authenticated=kwargs.pop('authenticated', True),
+            )
 
         # Create a dummy flask app to work with
         app = flask.Flask(__name__)
@@ -809,13 +895,12 @@ class TestCase(BaseTestCase):
             group='cache',
             backend='dogpile.cache.memory',
             enabled=True,
-            proxies=['oslo_cache.testing.CacheIsolatingProxy'])
+            proxies=['oslo_cache.testing.CacheIsolatingProxy'],
+        )
+        self.config_fixture.config(group='catalog', driver='sql')
         self.config_fixture.config(
-            group='catalog',
-            driver='sql',
-            template_file=dirs.tests('default_catalog.templates'))
-        self.config_fixture.config(
-            group='saml', certfile=signing_certfile, keyfile=signing_keyfile)
+            group='saml', certfile=signing_certfile, keyfile=signing_keyfile
+        )
         self.config_fixture.config(
             default_log_levels=[
                 'amqp=WARN',
@@ -831,7 +916,8 @@ class TestCase(BaseTestCase):
                 'stevedore.extension=INFO',
                 'keystone.notifications=INFO',
                 'keystone.identity.backends.ldap.common=INFO',
-            ])
+            ]
+        )
         # NOTE(notmorgan): Set password rounds low here to ensure speedy
         # tests. This is explicitly set because the tests here are not testing
         # the integrity of the password hashing, just that the correct form
@@ -843,7 +929,7 @@ class TestCase(BaseTestCase):
             ksfixtures.KeyRepository(
                 self.config_fixture,
                 'fernet_tokens',
-                CONF.fernet_tokens.max_active_keys
+                CONF.fernet_tokens.max_active_keys,
             )
         )
 
@@ -851,7 +937,7 @@ class TestCase(BaseTestCase):
             ksfixtures.KeyRepository(
                 self.config_fixture,
                 'fernet_receipts',
-                CONF.fernet_receipts.max_active_keys
+                CONF.fernet_receipts.max_active_keys,
             )
         )
 
@@ -859,7 +945,7 @@ class TestCase(BaseTestCase):
         assert self.__config_overrides_called is True
 
     def setUp(self):
-        super(TestCase, self).setUp()
+        super().setUp()
         self.__config_overrides_called = False
         self.__load_backends_called = False
         self.config_fixture = self.useFixture(config_fixture.Config(CONF))
@@ -871,9 +957,14 @@ class TestCase(BaseTestCase):
         # cleanup.
         def mocked_register_auth_plugin_opt(conf, opt):
             self.config_fixture.register_opt(opt, group='auth')
-        self.useFixture(fixtures.MockPatchObject(
-            keystone.conf.auth, '_register_auth_plugin_opt',
-            new=mocked_register_auth_plugin_opt))
+
+        self.useFixture(
+            fixtures.MockPatchObject(
+                keystone.conf.auth,
+                '_register_auth_plugin_opt',
+                new=mocked_register_auth_plugin_opt,
+            )
+        )
 
         self.config_overrides()
         # explicitly load auth configuration
@@ -931,31 +1022,37 @@ class TestCase(BaseTestCase):
 
         # TODO(termie): doing something from json, probably based on Django's
         #               loaddata will be much preferred.
-        if (hasattr(self, 'identity_api') and
-            hasattr(self, 'assignment_api') and
-                hasattr(self, 'resource_api')):
+        if (
+            hasattr(self, 'identity_api')
+            and hasattr(self, 'assignment_api')
+            and hasattr(self, 'resource_api')
+        ):
             try:
                 PROVIDERS.resource_api.create_domain(
-                    resource_base.NULL_DOMAIN_ID, fixtures.ROOT_DOMAIN)
+                    resource_base.NULL_DOMAIN_ID, fixtures.ROOT_DOMAIN
+                )
             except exception.Conflict:
                 # the root domain already exists, skip now.
                 pass
             for domain in fixtures.DOMAINS:
                 rv = PROVIDERS.resource_api.create_domain(domain['id'], domain)
-                attrname = 'domain_%s' % domain['id']
+                attrname = 'domain_{}'.format(domain['id'])
                 setattr(self, attrname, rv)
                 fixtures_to_cleanup.append(attrname)
 
             for project in fixtures.PROJECTS:
-                project_attr_name = 'project_%s' % project['name'].lower()
+                project_attr_name = 'project_{}'.format(
+                    project['name'].lower()
+                )
                 rv = PROVIDERS.resource_api.create_project(
-                    project['id'], project)
+                    project['id'], project
+                )
                 setattr(self, project_attr_name, rv)
                 fixtures_to_cleanup.append(project_attr_name)
 
             for role in fixtures.ROLES:
                 rv = PROVIDERS.role_api.create_role(role['id'], role)
-                attrname = 'role_%s' % role['name']
+                attrname = 'role_{}'.format(role['name'])
                 setattr(self, attrname, rv)
                 fixtures_to_cleanup.append(attrname)
 
@@ -973,12 +1070,13 @@ class TestCase(BaseTestCase):
                 # fixtures.ROLES[2] is the _member_ role.
                 for project_id in projects:
                     PROVIDERS.assignment_api.add_role_to_user_and_project(
-                        user_copy['id'], project_id, fixtures.ROLES[2]['id'])
+                        user_copy['id'], project_id, fixtures.ROLES[2]['id']
+                    )
 
                 # Use the ID from the fixture as the attribute name, so
                 # that our tests can easily reference each user dict, while
                 # the ID in the dict will be the real public ID.
-                attrname = 'user_%s' % user['name']
+                attrname = 'user_{}'.format(user['name'])
                 setattr(self, attrname, user_copy)
                 fixtures_to_cleanup.append(attrname)
 
@@ -986,9 +1084,10 @@ class TestCase(BaseTestCase):
                 role_id = role_assignment['role_id']
                 user = role_assignment['user']
                 project_id = role_assignment['project_id']
-                user_id = getattr(self, 'user_%s' % user)['id']
+                user_id = getattr(self, f'user_{user}')['id']
                 PROVIDERS.assignment_api.add_role_to_user_and_project(
-                    user_id, project_id, role_id)
+                    user_id, project_id, role_id
+                )
 
             self.addCleanup(self.cleanup_instance(*fixtures_to_cleanup))
 
@@ -1001,7 +1100,7 @@ class TestCase(BaseTestCase):
             # Short-circuit if the values are the same.
             return
 
-        msg = '%s != %s within %s delta' % (a, b, delta)
+        msg = f'{a} != {b} within {delta} delta'
 
         self.assertLessEqual(abs(a - b).seconds, delta, msg)
 
@@ -1011,10 +1110,7 @@ class TestCase(BaseTestCase):
         # is working to eliminate microseconds from it's datetimes used.
         expected = timeutils.parse_isotime(expected).replace(microsecond=0)
         value = timeutils.parse_isotime(value).replace(microsecond=0)
-        self.assertEqual(
-            expected,
-            value,
-            "%s != %s" % (expected, value))
+        self.assertEqual(expected, value, f"{expected} != {value}")
 
     def assertNotEmpty(self, iterable):
         self.assertGreater(len(iterable), 0)
@@ -1030,8 +1126,11 @@ class TestCase(BaseTestCase):
         # NOTE(notmorgan): An empty option list is the same as no options being
         # specified in the user_ref. This removes options if it is empty in
         # observed if options is not specified in the expected value.
-        if ('options' in observed and not observed['options'] and
-                'options' not in expected):
+        if (
+            'options' in observed
+            and not observed['options']
+            and 'options' not in expected
+        ):
             observed = observed.copy()
             del observed['options']
 
@@ -1046,7 +1145,7 @@ class TestCase(BaseTestCase):
                 # NOTE(Mouad): Try to bind to IPv6 loopback ip address.
                 sock.bind(("::1", 0))
                 return True
-            except socket.error:
+            except OSError:
                 pass
             finally:
                 if sock:
@@ -1058,11 +1157,11 @@ class TestCase(BaseTestCase):
             raise self.skipTest("IPv6 is not enabled in the system")
 
 
-class SQLDriverOverrides(object):
+class SQLDriverOverrides:
     """A mixin for consolidating sql-specific test overrides."""
 
     def config_overrides(self):
-        super(SQLDriverOverrides, self).config_overrides()
+        super().config_overrides()
         # SQL specific driver overrides
         self.config_fixture.config(group='catalog', driver='sql')
         self.config_fixture.config(group='identity', driver='sql')
