@@ -24,13 +24,18 @@ class LDAPDatabase(fixtures.Fixture):
         self._dbclass = dbclass
 
     def setUp(self):
-        super(LDAPDatabase, self).setUp()
+        super().setUp()
         self.clear()
+        common_ldap.WRITABLE = True
         common_ldap._HANDLERS.clear()
         common_ldap.register_handler('fake://', self._dbclass)
         # TODO(dstanek): switch the flow here
         self.addCleanup(self.clear)
         self.addCleanup(common_ldap._HANDLERS.clear)
+        self.addCleanup(self.disable_write)
+
+    def disable_write(self):
+        common_ldap.WRITABLE = False
 
     def clear(self):
         for shelf in fakeldap.FakeShelves:
