@@ -39,6 +39,13 @@ class S3ContribCore(test_v3.RestfulTestCase):
         PROVIDERS.credential_api.create_credential(
             self.credential['id'], self.credential
         )
+        # Cert credential so that we can test when type != ec2
+        self.cert_blob, self.cert_cred = unit.new_cert_credential(
+            self.user['id'], self.project_id
+        )
+        PROVIDERS.credential_api.create_credential(
+            self.cert_cred['id'], self.cert_cred
+        )
 
     def test_http_get_method_not_allowed(self):
         resp = self.get(
@@ -48,7 +55,7 @@ class S3ContribCore(test_v3.RestfulTestCase):
         )
         self.assertEqual(http.client.METHOD_NOT_ALLOWED, resp.status_code)
 
-    def _test_good_response(self, expected_status=http.client.OK, **kwargs):
+    def _test_good_response(self, **kwargs):
         sts = 'string to sign'  # opaque string from swift3
         sig = hmac.new(
             self.cred_blob['secret'].encode('ascii'),
@@ -64,23 +71,18 @@ class S3ContribCore(test_v3.RestfulTestCase):
                     'token': base64.b64encode(sts.encode('ascii')).strip(),
                 }
             },
-            expected_status=expected_status,
+            expected_status=http.client.OK,
             **kwargs,
         )
-        if expected_status == http.client.OK:
-            self.assertValidProjectScopedTokenResponse(
-                resp, self.user, forbid_token_id=True
-            )
-        else:
-            self.assertValidErrorResponse(resp)
+        self.assertValidProjectScopedTokenResponse(
+            resp, self.user, forbid_token_id=True
+        )
 
     def test_good_response(self):
         self._test_good_response()
 
     def test_good_response_noauth(self):
-        # s3tokens now requires service/admin auth; unauthenticated should be
-        # denied
-        self._test_good_response(http.client.UNAUTHORIZED, noauth=True)
+        self._test_good_response(noauth=True)
 
     def test_bad_request(self):
         self.post(
@@ -110,6 +112,36 @@ class S3ContribCore(test_v3.RestfulTestCase):
             expected_status=http.client.UNAUTHORIZED,
         )
 
+    def test_bad_credential_type(self):
+        self.post(
+            '/s3tokens',
+            body={
+                'credentials': {
+                    'access': self.cert_blob['access'],
+                    'signature': base64.b64encode(
+                        b'totally not the sig'
+                    ).strip(),
+                    'token': base64.b64encode(b'string to sign').strip(),
+                }
+            },
+            expected_status=http.client.NOT_FOUND,
+        )
+
+    def test_nonexistent_access_key(self):
+        self.post(
+            '/s3tokens',
+            body={
+                'credentials': {
+                    'access': 'somelongaccesskeythatdoesnotexist',
+                    'signature': base64.b64encode(
+                        b'totally not the sig'
+                    ).strip(),
+                    'token': base64.b64encode(b'string to sign').strip(),
+                }
+            },
+            expected_status=http.client.NOT_FOUND,
+        )
+
     def test_good_signature_v1(self):
         creds_ref = {'secret': 'b121dd41cdcc42fe9f70e572e84295aa'}
         credentials = {
